<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once("Secure_Controller.php");

class Receivings extends Secure_Controller
{
	public function __construct()
	{
		parent::__construct('receivings');

		$this->load->library('receiving_lib');
		$this->load->library('barcode_lib');

		$this->load->library('item_lib');
		$config_data = array(
			'value' => 25
		);

		$this->db->where('key', 'lines_per_page');
		$success = $this->db->update('ospos_app_config', $config_data);
	}

	public function index()
	{
		$this->_reload();
	}

	public function item_search()
	{
		$suggestions = $this->Item->get_search_suggestions($this->input->get('term'), array('search_custom' => FALSE, 'is_deleted' => FALSE), TRUE);
		$suggestions = array_merge($suggestions, $this->Item_kit->get_search_suggestions($this->input->get('term')));

		$suggestions = $this->xss_clean($suggestions);

		echo json_encode($suggestions);
	}

	public function select_supplier()
	{
		$supplier_id = $this->input->post('supplier');
		if($this->Supplier->exists($supplier_id))
		{
			$this->receiving_lib->set_supplier($supplier_id);
		}

		$this->_reload();
	}

	public function change_mode()
	{
		$stock_destination = $this->input->post('stock_destination');
		$stock_source = $this->input->post('stock_source');

		if((!$stock_source || $stock_source == $this->receiving_lib->get_stock_source()) &&
			(!$stock_destination || $stock_destination == $this->receiving_lib->get_stock_destination()))
		{
			$this->receiving_lib->clear_reference();
			$mode = $this->input->post('mode');
			$this->receiving_lib->set_mode($mode);
		}
		elseif($this->Stock_location->is_allowed_location($stock_source, 'receivings'))
		{
			$this->receiving_lib->set_stock_source($stock_source);
			$this->receiving_lib->set_stock_destination($stock_destination);
		}

		$this->_reload();
	}
	
	public function set_comment()
	{
		$this->receiving_lib->set_comment($this->input->post('comment'));
	}

	public function set_print_after_sale()
	{
		$this->receiving_lib->set_print_after_sale($this->input->post('recv_print_after_sale'));
	}
	
	public function set_reference()
	{
		$this->receiving_lib->set_reference($this->input->post('recv_reference'));
	}
	
	public function add()
	{
		$data = array();

		$mode = $this->receiving_lib->get_mode();
		$item_id_or_number_or_item_kit_or_receipt = $this->input->post('item');
		$quantity = ($mode == 'receive' || $mode == 'requisition') ? 1 : -1;
		$item_location = $this->receiving_lib->get_stock_source();
		

		if($mode == 'return' && $this->Receiving->is_valid_receipt($item_id_or_number_or_item_kit_or_receipt))
		{
			$this->receiving_lib->return_entire_receiving($item_id_or_number_or_item_kit_or_receipt);
		}
		elseif($this->Item_kit->is_valid_item_kit($item_id_or_number_or_item_kit_or_receipt))
		{
			$this->receiving_lib->add_item_kit($item_id_or_number_or_item_kit_or_receipt, $item_location);
		}
		elseif(!$this->receiving_lib->add_item($item_id_or_number_or_item_kit_or_receipt, $quantity, $item_location))
		{
			$data['error'] = $this->lang->line('receivings_unable_to_add_item');
		}

		$this->_reload($data);
	}

	public function edit_item($item_id)
	{
		$data = array();

		$this->form_validation->set_rules('price', 'lang:items_price', 'required|callback_numeric');
		$this->form_validation->set_rules('quantity', 'lang:items_quantity', 'required|callback_numeric');
		$this->form_validation->set_rules('discount', 'lang:items_discount', 'required|callback_numeric');

		$description = $this->input->post('description');
		$serialnumber = $this->input->post('serialnumber');
		$price = parse_decimals($this->input->post('price'));
		$quantity = parse_decimals($this->input->post('quantity'));
		$discount = parse_decimals($this->input->post('discount'));
		$item_location = $this->input->post('location');
		$expiry_date = $this->input->post('expiry_date');

		if($this->form_validation->run() != FALSE)
		{
			$this->receiving_lib->edit_item($item_id, $description, $serialnumber, $quantity, $discount, $price,$expiry_date);
		}
		else
		{
			$data['error']=$this->lang->line('receivings_error_editing_item');
		}

		$this->_reload($data);
	}
	
	public function edit($receiving_id)
	{
		$data = array();

		$data['suppliers'] = array('' => 'No Supplier');
		foreach($this->Supplier->get_all()->result() as $supplier)
		{
			$data['suppliers'][$supplier->person_id] = $this->xss_clean($supplier->first_name . ' ' . $supplier->last_name);
		}
	
		$data['employees'] = array();
		foreach ($this->Employee->get_all()->result() as $employee)
		{
			$data['employees'][$employee->person_id] = $this->xss_clean($employee->first_name . ' '. $employee->last_name);
		}
	
		$receiving_info = $this->xss_clean($this->Receiving->get_info($receiving_id)->row_array());
		$data['selected_supplier_name'] = !empty($receiving_info['supplier_id']) ? $receiving_info['company_name'] : '';
		$data['selected_supplier_id'] = $receiving_info['supplier_id'];
		$data['receiving_info'] = $receiving_info;
	
		$this->load->view('receivings/form', $data);
	}

	public function delete_item($item_number)
	{
		$this->receiving_lib->delete_item($item_number);

		$this->_reload();
	}
	
	public function delete($receiving_id = -1, $update_inventory = TRUE) 
	{
		$employee_id = $this->Employee->get_logged_in_employee_info()->person_id;
		$receiving_ids = $receiving_id == -1 ? $this->input->post('ids') : array($receiving_id);
	
		if($this->Receiving->delete_list($receiving_ids, $employee_id, $update_inventory))
		{
			echo json_encode(array('success' => TRUE, 'message' => $this->lang->line('receivings_successfully_deleted') . ' ' .
							count($receiving_ids) . ' ' . $this->lang->line('receivings_one_or_multiple'), 'ids' => $receiving_ids));
		}
		else
		{
			echo json_encode(array('success' => FALSE, 'message' => $this->lang->line('receivings_cannot_be_deleted')));
		}
	}

	public function remove_supplier()
	{
		$this->receiving_lib->clear_reference();
		$this->receiving_lib->remove_supplier();

		$this->_reload();
	}

	public function complete()
	{
		$data = array();
		
		$data['cart'] = $this->receiving_lib->get_cart();
		$data['total'] = $this->receiving_lib->get_total();
		$data['receipt_title'] = $this->lang->line('receivings_receipt');
		$data['transaction_time'] = date($this->config->item('dateformat') . ' ' . $this->config->item('timeformat'));
		$data['mode'] = $this->receiving_lib->get_mode();
		$data['comment'] = $this->receiving_lib->get_comment();
		$data['reference'] = $this->receiving_lib->get_reference();
		$data['payment_type'] = $this->input->post('payment_type');
		$data['show_stock_locations'] = $this->Stock_location->show_locations('receivings');
		$data['stock_location'] = $this->receiving_lib->get_stock_source();
		if($this->input->post('amount_tendered') != NULL)
		{
			$data['amount_tendered'] = $this->input->post('amount_tendered');
			$data['amount_change'] = to_currency($data['amount_tendered'] - $data['total']);
		}
		
		$employee_id = $this->Employee->get_logged_in_employee_info()->person_id;
		$employee_info = $this->Employee->get_info($employee_id);
		$data['employee'] = $employee_info->first_name . ' ' . $employee_info->last_name;

		$supplier_info = '';
		$supplier_id = $this->receiving_lib->get_supplier();
		if($supplier_id != -1)
		{
			$supplier_info = $this->Supplier->get_info($supplier_id);
			$data['supplier'] = $supplier_info->company_name;
			$data['first_name'] = $supplier_info->first_name;
			$data['last_name'] = $supplier_info->last_name;
			$data['supplier_email'] = $supplier_info->email;
			$data['supplier_address'] = $supplier_info->address_1;
			if(!empty($supplier_info->zip) or !empty($supplier_info->city))
			{
				$data['supplier_location'] = $supplier_info->zip . ' ' . $supplier_info->city;				
			}
			else
			{
				$data['supplier_location'] = '';
			}
		}

		//SAVE receiving to database
		$data['receiving_id'] = 'RECV ' . $this->Receiving->save($data['cart'], $supplier_id, $employee_id, $data['comment'], $data['reference'], $data['payment_type'], $data['stock_location']);

		$data = $this->xss_clean($data);

		if($data['receiving_id'] == 'RECV -1')
		{
			$data['error_message'] = $this->lang->line('receivings_transaction_failed');
		}
		else
		{
			$data['barcode'] = $this->barcode_lib->generate_receipt_barcode($data['receiving_id']);				
		}

		$data['print_after_sale'] = $this->receiving_lib->is_print_after_sale();

		$this->load->view("receivings/receipt",$data);

		$this->receiving_lib->clear_all();
	}

	public function requisition_complete()
	{
		if($this->receiving_lib->get_stock_source() != $this->receiving_lib->get_stock_destination()) 
		{
			foreach($this->receiving_lib->get_cart() as $item)
			{
				$this->receiving_lib->delete_item($item['line']);
				$this->receiving_lib->add_item($item['item_id'], $item['quantity'], $this->receiving_lib->get_stock_destination());
				$this->receiving_lib->add_item($item['item_id'], -$item['quantity'], $this->receiving_lib->get_stock_source());
			}
			
			$this->complete();
		}
		else 
		{
			$data['error'] = $this->lang->line('receivings_error_requisition');

			$this->_reload($data);	
		}
	}
	
	public function receipt($receiving_id)
	{
		$receiving_info = $this->Receiving->get_info($receiving_id)->row_array();
		$this->receiving_lib->copy_entire_receiving($receiving_id);
		$data['cart'] = $this->receiving_lib->get_cart();
		$data['total'] = $this->receiving_lib->get_total();
		$data['mode'] = $this->receiving_lib->get_mode();
		$data['receipt_title'] = $this->lang->line('receivings_receipt');
		$data['transaction_time'] = date($this->config->item('dateformat') . ' ' . $this->config->item('timeformat'), strtotime($receiving_info['receiving_time']));
		$data['show_stock_locations'] = $this->Stock_location->show_locations('receivings');
		$data['payment_type'] = $receiving_info['payment_type'];
		$data['reference'] = $this->receiving_lib->get_reference();
		$data['receiving_id'] = 'RECV ' . $receiving_id;
		$data['barcode'] = $this->barcode_lib->generate_receipt_barcode($data['receiving_id']);
		$employee_info = $this->Employee->get_info($receiving_info['employee_id']);
		$data['employee'] = $employee_info->first_name . ' ' . $employee_info->last_name;

		$supplier_id = $this->receiving_lib->get_supplier();
		if($supplier_id != -1)
		{
			$supplier_info = $this->Supplier->get_info($supplier_id);
			$data['supplier'] = $supplier_info->company_name;
			$data['first_name'] = $supplier_info->first_name;
			$data['last_name'] = $supplier_info->last_name;
			$data['supplier_email'] = $supplier_info->email;
			$data['supplier_address'] = $supplier_info->address_1;
			if(!empty($supplier_info->zip) or !empty($supplier_info->city))
			{
				$data['supplier_location'] = $supplier_info->zip . ' ' . $supplier_info->city;				
			}
			else
			{
				$data['supplier_location'] = '';
			}
		}

		$data['print_after_sale'] = FALSE;

		$data = $this->xss_clean($data);
		
		$this->load->view("receivings/receipt", $data);

		$this->receiving_lib->clear_all();
	}

	private function _reload($data = array())
	{
		$data['cart'] = $this->receiving_lib->get_cart();
		$data['modes'] = array('receive' => $this->lang->line('receivings_receiving'), 'return' => $this->lang->line('receivings_return'));
		$data['mode'] = $this->receiving_lib->get_mode();
		$data['stock_locations'] = $this->Stock_location->get_allowed_locations('receivings');
		$data['show_stock_locations'] = count($data['stock_locations']) > 1;
		if($data['show_stock_locations']) 
		{
			$data['modes']['requisition'] = $this->lang->line('receivings_requisition');
			$data['stock_source'] = $this->receiving_lib->get_stock_source();
			$data['stock_destination'] = $this->receiving_lib->get_stock_destination();
		}

		$data['total'] = $this->receiving_lib->get_total();
		$data['items_module_allowed'] = $this->Employee->has_grant('items', $this->Employee->get_logged_in_employee_info()->person_id);
		$data['comment'] = $this->receiving_lib->get_comment();
		$data['reference'] = $this->receiving_lib->get_reference();
		$data['payment_options'] = $this->Receiving->get_payment_options();

		$supplier_id = $this->receiving_lib->get_supplier();
		$supplier_info = '';
		if($supplier_id != -1)
		{
			$supplier_info = $this->Supplier->get_info($supplier_id);
			$data['supplier'] = $supplier_info->company_name;
			$data['first_name'] = $supplier_info->first_name;
			$data['last_name'] = $supplier_info->last_name;
			$data['supplier_email'] = $supplier_info->email;
			$data['supplier_address'] = $supplier_info->address_1;
			if(!empty($supplier_info->zip) or !empty($supplier_info->city))
			{
				$data['supplier_location'] = $supplier_info->zip . ' ' . $supplier_info->city;				
			}
			else
			{
				$data['supplier_location'] = '';
			}
		}
		
		$data['print_after_sale'] = $this->receiving_lib->is_print_after_sale();

		$data['habilitacion_caja'] = $this->Sale->get_habilitacion_caja($this->Employee->get_logged_in_employee_info()->person_id);

		$data = $this->xss_clean($data);

		$this->load->view("receivings/receiving", $data);
	}
	
	public function save($receiving_id = -1)
	{
		$newdate = $this->input->post('date');
		
		$date_formatter = date_create_from_format($this->config->item('dateformat') . ' ' . $this->config->item('timeformat'), $newdate);

		$receiving_data = array(
			'receiving_time' => $date_formatter->format('Y-m-d H:i:s'),
			'supplier_id' => $this->input->post('supplier_id') ? $this->input->post('supplier_id') : NULL,
			'employee_id' => $this->input->post('employee_id'),
			'comment' => $this->input->post('comment'),
			'reference' => $this->input->post('reference') != '' ? $this->input->post('reference') : NULL
		);
	
		if($this->Receiving->update($receiving_data, $receiving_id))
		{
			echo json_encode(array('success' => TRUE, 'message' => $this->lang->line('receivings_successfully_updated'), 'id' => $receiving_id));
		}
		else
		{
			echo json_encode(array('success' => FALSE, 'message' => $this->lang->line('receivings_unsuccessfully_updated'), 'id' => $receiving_id));
		}
	}

	public function cancel_receiving()
	{
		$this->receiving_lib->clear_all();

		$this->_reload();
	}
	/*Producción*/
	public function production()
	{

		
		$formula_id = $this->receiving_lib->get_formula();
		$production_quantity = $this->receiving_lib->get_production_quantity();
		$data['formula'] = $formula_id;
		$data['production_quantity'] = $production_quantity;
		$items = $this->Item->get_formulas_items($formula_id);
	   	$data['items']=$items;
	   	$data['workers'] = $this->receiving_lib->get_workers();
	   	$formulas_row = $this->Item->get_formulas_row($formula_id);
	   	foreach($formulas_row->result_array() as $formula)
		{
			$data['description'] = $formula['description'];
			$data['item_number'] = $formula['item_number']; 
			$data['item_id'] = $formula['item_id'];
			$data['name'] = $formula['name'];
			$data['quantity'] = $formula['quantity'];
			$data['unit_measurement'] = $formula['unit_measurement'];
		}


		$this->load->view("receivings/production", $data);
	}

	public function set_formula()
	{	
		$formula_id = $this->input->get('formula_id');
		$this->receiving_lib->set_formula($formula_id);
		$this->production();
	}

	public function save_production()
	{
		$formula_id = $this->input->get('formula_id');
		$cantidad = $this->input->get('quantity');
		$comment = $this->input->get('description');
		$cant_producir = $this->input->get('cant_producir');
		$item_id = $this->input->get('item_id');
		$total_costo_produccion = $this->input->get('total_costo_produccion');
		
		$employee_id = $this->Employee->get_logged_in_employee_info()->person_id;
		$items = $this->Item->get_formulas_items($formula_id);
		
		$receivings_data = array(
			'receiving_time' => date('Y-m-d H:i:s'),
			'supplier_id' => NULL,
			'employee_id' => $employee_id,
			'payment_type' => 'No aplica',
			'comment' => 'PRODUCCION',
			'reference' => '' //Concatenar numero de producción
		);


		//Run these queries as a transaction, we want to make sure we do all or nothing
		$this->db->trans_start();

		$this->db->insert('receivings', $receivings_data);
		$receiving_id = $this->db->insert_id();


		$productions_data = array(
			'production_time' => date('Y-m-d H:i:s'),
			'employee_id' => $employee_id,
			'comment' => $comment,
			'receiving_id' => $receiving_id
		);
		$this->db->insert('productions', $productions_data);
		$production_id = $this->db->insert_id();

		$line = 0;
		foreach($items->result_array() as $item)
		{
			
			$cur_item_info = $this->Item->get_info($item['item_id']);
			$codigo = $this->Item->get_item_number($item['item_id']);
			$categoria = $this->Item->get_category($codigo);
			$item_idx = $item['item_id'];
			$item_location = 1;
			$receiving_quantity = 0;
			$costox = $item['price'];
			$costo = 0;
			
			if ($categoria == 'GASTOS') {
				$costo = 0;
			}else
			{
				$costo = $cur_item_info->cost_price;
			}
			$line = $line +1;
			/*echo "string ".$receiving_id."-".$item['item_id']."-".$line."-".$cur_item_info->description." q ".$item['quantity']."-".$costo."-".$item_location;*/
			
			$receivings_items_data = array(
				'receiving_id' => $receiving_id,
				'item_id' => $item['item_id'],
				'line' => $line,
				'description' => $cur_item_info->description,
				'serialnumber' => NULL,
				'quantity_purchased' => $item['quantity']*$cantidad *-1,
				'receiving_quantity' => $receiving_quantity,
				'discount_percent' => 0,
				'item_cost_price' => $costo,
				'item_unit_price' => $costo,
				'item_location' => $item_location //editar en la pantalla de producción
			);

			$this->db->insert('receivings_items', $receivings_items_data);
			//die();


			/*Detalle producción*/

			$productions_items_data = array(
				'production_id' => $production_id,
				'item_id' => $item['item_id'],
				'quantity' => $item['quantity']*$cantidad,
				'item_location' => $item_location,
				'cost_price' => $costo
				 //editar en la pantalla de producción
			);
			$this->db->insert('productions_items', $productions_items_data);

			/*Fin detalle producción*/

			$items_received = $receiving_quantity != 0 ? $item['quantity'] * $receiving_quantity : $item['quantity']*$cantidad *-1;/*Se multiplica por -1 porque hay que restar las materias primas*/

			// update cost price, if changed AND is set in config as wanted
			if($cur_item_info->cost_price != $costo && $this->config->item('receiving_calculate_average_price') != FALSE)
			{
				$this->Item->change_cost_price($item['item_id'], $items_received, $costo, $cur_item_info->cost_price);
			}

			//Update stock quantity
			$item_quantity = $this->Item_quantity->get_item_quantity($item['item_id'], $item_location);
            $this->Item_quantity->save(array('quantity' => $item_quantity->quantity + $items_received, 'item_id' => $item['item_id'],
                                              'location_id' => $item_location), $item['item_id'], $item_location);

			$recv_remarks = 'RECV ' . $receiving_id.' PROD '.$production_id;
			$inv_data = array(
				'trans_date' => date('Y-m-d H:i:s'),
				'trans_items' => $item['item_id'],
				'trans_user' => $employee_id,
				'trans_location' => $item_location,//$item['item_location'],
				'trans_comment' => $recv_remarks,
				'trans_inventory' => $items_received
			);

			$this->Inventory->insert($inv_data);

			$supplier = $this->Supplier->get_info($supplier_id);

			//Actualizar costo
			//Verificar si es compra para modificar el costo
			/*if ($item['quantity']>0) {
				$item_datax = array(
				'cost_price' => $costox
				);
				$this->db->where('item_id', $item_idx);
				$success2 = $this->db->update('ospos_items', $item_datax);
				//Fin Actualiza costo
			}*/
			//Fin Actualiza costo
		}

		/*PRODUCTO FINALIZADO*/
		$line = $line + 1;
		$costo_total = $total_costo_produccion/$cant_producir;
		$receivings_items_data = array(
				'receiving_id' => $receiving_id,
				'item_id' => $item_id,
				'line' => $line,
				'description' => $comment,
				'serialnumber' => NULL,
				'quantity_purchased' => $cant_producir,
				'receiving_quantity' => $receiving_quantity,
				'discount_percent' => 0,
				'item_cost_price' => $costo_total,
				'item_unit_price' => $costo_total,
				'item_location' => $item_location //editar en la pantalla de producción
			);

			$this->db->insert('receivings_items', $receivings_items_data);
			//die();


			/*Detalle producción FINISHED*/

			$productions_finished_data = array(
				'production_id' => $production_id,
				'item_id' => $item_id,
				'quantity' => $cant_producir,
				'item_location' => $item_location,
				'cost_price' => $costo_total
				 //editar en la pantalla de producción
			);
			$this->db->insert('productions_finished', $productions_finished_data);

			/*Fin detalle producción FINISHED*/

			$items_received = $receiving_quantity != 0 ? $cant_producir * $receiving_quantity : $cant_producir;/*Se multiplica por -1 porque hay que restar las materias primas*/

			// update cost price, if changed AND is set in config as wanted
			if($cur_item_info->cost_price != $costo && $this->config->item('receiving_calculate_average_price') != FALSE)
			{
				$this->Item->change_cost_price($item_id, $items_received, $costo, $cur_item_info->cost_price);
			}

			//Update stock quantity
			$item_quantity = $this->Item_quantity->get_item_quantity($item_id, $item_location);
            $this->Item_quantity->save(array('quantity' => $item_quantity->quantity + $items_received, 'item_id' => $item_id,
                                              'location_id' => $item_location), $item_id, $item_location);

			$recv_remarks = 'RECV ' . $receiving_id.' PROD '.$production_id;
			$inv_data = array(
				'trans_date' => date('Y-m-d H:i:s'),
				'trans_items' => $item_id,
				'trans_user' => $employee_id,
				'trans_location' => $item_location,//$item['item_location'],
				'trans_comment' => $recv_remarks,
				'trans_inventory' => $items_received
			);

			$this->Inventory->insert($inv_data);

		/*FIN PRODUCTO FINALIZADO*/
		
		/*EMPLEADOS PRODUCCION OPERARIOS*/
		$workers = $this->receiving_lib->get_workers();
		foreach($workers as $worker)
		{
			
			$productions_employees_data = array(
				'production_id' => $production_id,
				'employee_id' => $worker['worker_id']
			);
			$this->db->insert('productions_employees', $productions_employees_data);
		}

		/*EMPLEADOS PRODUCCION OPERARIOS*/

		$this->db->trans_complete();
		
		
		if($this->db->trans_status() === FALSE)
		{
			$data['error'] = 'No se guardó el movimiento';
		}
		else
		{
			$data['success'] = 'Se generó el movimiento nro. '.$receiving_id;
		}
		//return $receiving_id;
		$formula_id = 0;//$this->receiving_lib->get_formula();
		$production_quantity = 0;//$this->receiving_lib->get_production_quantity();
		$data['formula'] = $formula_id;
		$data['production_quantity'] = $production_quantity;
		$items = $this->Item->get_formulas_items($formula_id);
	   	$data['items']=$items;

		$this->load->view("receivings/production", $data);
	}

	public function add_worker(){

		$worker_id = $this->input->get('worker_id');
		$insertkey = $worker_id;
		$workers = $this->receiving_lib->get_workers();

		//var_dump($workers);
		//echo "<p>";
		if($this->Employee->exists($worker_id)){
			$name = $this->Person->get_name($worker_id);
			
			$worker = array($insertkey => array(
                    'worker_id' => $worker_id,
                    'name' => $name
                )
            );
			//add to existing array
			$workers += $worker;
			$this->receiving_lib->set_workers($workers);
		}
		$workers = $this->receiving_lib->get_workers();
		//var_dump($workers);

		/*$formula_id = $this->receiving_lib->get_formula();
		$production_quantity = $this->receiving_lib->get_production_quantity();
		$data['formula'] = $formula_id;
		$data['production_quantity'] = $production_quantity;
		$items = $this->Item->get_formulas_items($formula_id);
	   	$data['items']=$items;
	   	$data['workers'] = $workers;

		$this->load->view("receivings/production", $data);*/

		$this->production();
	}

	public function delete_worker(){

		$worker_id = $this->input->get('worker_id');
		$insertkey = $worker_id;
		$workers = $this->receiving_lib->get_workers();

		unset($workers[$insertkey]);

		$this->receiving_lib->set_workers($workers);

		$workers = $this->receiving_lib->get_workers();
		//var_dump($workers);

		/*$formula_id = $this->receiving_lib->get_formula();
		$data['formula'] = $formula_id;
		$items = $this->Item->get_formulas_items($formula_id);
	   	$data['items']=$items;
	   	$data['workers'] = $workers;

		$this->load->view("receivings/production", $data);*/
		$this->production();
	}


	public function set_production_quantity()
	{	
		$production_quantity = $this->input->get('production_quantity');
		$this->receiving_lib->set_production_quantity($production_quantity);
		$this->production();
	}

	/*Fin Producción*/
}
?>
