<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once("Secure_Controller.php");

class Sales extends Secure_Controller
{
	public function __construct()
	{
		parent::__construct('sales');

		$this->load->library('sale_lib');
		$this->load->library('barcode_lib');
		$this->load->library('email_lib');

		$this->load->library('item_lib');
		$config_data = array(
			'value' => 10000
		);

		$this->db->where('key', 'lines_per_page');
		$success = $this->db->update('ospos_app_config', $config_data);
	}

	public function index()
	{
		$this->_reload();
	}
	
	public function manage()
	{
		$person_id = $this->session->userdata('person_id');
		//reports_inventory
		//if(!$this->Employee->has_grant('reports_sales', $person_id))
		if(!$this->Employee->has_grant('reports_inventory', $person_id))
		{
			redirect('no_access/sales/reports_sales');
		}
		else
		{
			$data['table_headers'] = get_sales_manage_table_headers();

			// filters that will be loaded in the multiselect dropdown
			if($this->config->item('invoice_enable') == TRUE)
			{
				$data['filters'] = array('only_cash' => $this->lang->line('sales_cash_filter'),
										'only_invoices' => $this->lang->line('sales_invoice_filter'));
			}
			else
			{
				$data['filters'] = array('only_cash' => $this->lang->line('sales_cash_filter'));
			}

			$this->load->view('sales/manage', $data);
		}
	}
	
	public function get_row($row_id)
	{
		$sale_info = $this->Sale->get_info($row_id)->row();
		$data_row = $this->xss_clean(get_sale_data_row($sale_info, $this));

		echo json_encode($data_row);
	}

	public function search()
	{
		$search = $this->input->get('search');
		$limit  = $this->input->get('limit');
		$offset = $this->input->get('offset');
		$sort   = $this->input->get('sort');
		$order  = $this->input->get('order');

		$filters = array('sale_type' => 'all',
						'location_id' => 'all',
						'start_date' => $this->input->get('start_date'),
						'end_date' => $this->input->get('end_date'),
						'only_cash' => FALSE,
						'only_invoices' => $this->config->item('invoice_enable') && $this->input->get('only_invoices'),
						'is_valid_receipt' => $this->Sale->is_valid_receipt($search));

		// check if any filter is set in the multiselect dropdown
		$filledup = array_fill_keys($this->input->get('filters'), TRUE);
		$filters = array_merge($filters, $filledup);

		$sales = $this->Sale->search($search, $filters, $limit, $offset, $sort, $order);
		$total_rows = $this->Sale->get_found_rows($search, $filters);
		$payments = $this->Sale->get_payments_summary($search, $filters);
		$payment_summary = $this->xss_clean(get_sales_manage_payments_summary($payments, $sales, $this));

		$data_rows = array();

		foreach(array_reverse($sales->result(),true) as $sale)
		{
			$data_rows[] = $this->xss_clean(get_sale_data_row($sale, $this));
		}

		if($total_rows > 0)
		{
			$data_rows[] = $this->xss_clean(get_sale_data_last_row($sales, $this));
		}

		echo json_encode(array('total' => $total_rows, 'rows' => $data_rows, 'payment_summary' => $payment_summary));
	}

	public function item_search()
	{
		$suggestions = array();
		$receipt = $search = $this->input->get('term') != '' ? $this->input->get('term') : NULL;

		if($this->sale_lib->get_mode() == 'return' && $this->Sale->is_valid_receipt($receipt))
		{
			// if a valid receipt or invoice was found the search term will be replaced with a receipt number (POS #)
			$suggestions[] = $receipt;
		}
		$suggestions = array_merge($suggestions, $this->Item->get_search_suggestions($search, array('search_custom' => FALSE, 'is_deleted' => FALSE), TRUE));
		$suggestions = array_merge($suggestions, $this->Item_kit->get_search_suggestions($search));
		
		$suggestions = $this->xss_clean($suggestions);

		echo json_encode($suggestions);
	}

	public function suggest_search()
	{
		$search = $this->input->post('term') != '' ? $this->input->post('term') : NULL;
		
		$suggestions = $this->xss_clean($this->Sale->get_search_suggestions($search));
		
		echo json_encode($suggestions);
	}

	public function select_customer()
	{
		$customer_id = $this->input->post('customer');
		$this->sale_lib->set_ruc($customer_id);
		if($this->Customer->exists($customer_id))
		{
			$this->sale_lib->set_customer($customer_id);

			$discount_percent = $this->Customer->get_info($customer_id)->discount_percent;

			// apply customer default discount to items that have 0 discount
			if($discount_percent != '')
			{	
				$this->sale_lib->apply_customer_discount($discount_percent);
			}
			$this->sale_lib->set_invoice_number_enabled(TRUE);
			
		}

		
		$this->_reload();
	}

	public function select_customer_sn()
	{
		$customer_id = $this->input->post('customer_sn');
		if($this->Customer->exists($customer_id))
		{
			$this->sale_lib->set_customer($customer_id);

			$discount_percent = $this->Customer->get_info($customer_id)->discount_percent;

			// apply customer default discount to items that have 0 discount
			if($discount_percent != '')
			{	
				$this->sale_lib->apply_customer_discount($discount_percent);
			}
			$this->sale_lib->set_invoice_number_enabled(TRUE);
			
		}
		
		$this->_reload();
	}
	
	public function select_seller()
	{
		$employee_id = $this->input->post('seller');
		if($this->Employee->exists($employee_id))
		{
			$this->sale_lib->set_seller($employee_id);
		}
		
		$this->_reload();
	}

	public function change_mode()
	{
		$stock_location = $this->input->post('stock_location');
		if (!$stock_location || $stock_location == $this->sale_lib->get_sale_location())
		{
			$mode = $this->input->post('mode');
			$this->sale_lib->set_mode($mode);
		} 
		elseif($this->Stock_location->is_allowed_location($stock_location, 'sales'))
		{
			$this->sale_lib->set_sale_location($stock_location);
		}

		$this->_reload();
	}
	
	public function set_comment() 
	{
		$this->sale_lib->set_comment($this->input->post('comment'));
	}
	
	public function set_invoice_number()
	{
		$this->sale_lib->set_invoice_number($this->input->post('sales_invoice_number'));
	}

	public function set_invoice_number_enabled()
	{
		$this->sale_lib->set_invoice_number_enabled($this->input->post('sales_invoice_number_enabled'));
	}
	
	public function set_print_after_sale()
	{
		$this->sale_lib->set_print_after_sale($this->input->post('sales_print_after_sale'));
	}
	
	public function unset_print_after_sale()
	{
		$this->sale_lib->unset_print_after_sale();
	}

	public function set_invoice_number_enabled_custom()
	{
		$this->sale_lib->set_invoice_number_enabled(1);
	}

	public function set_email_receipt()
	{
 		$this->sale_lib->set_email_receipt($this->input->post('email_receipt'));
	}

	// Multiple Payments
	public function add_payment()
	{
		$data = array();
		$this->form_validation->set_rules('amount_tendered', 'lang:sales_amount_tendered', 'trim|required|callback_numeric');

		$payment_type = $this->input->post('payment_type');

		if($this->form_validation->run() == FALSE)
		{
			if($payment_type == $this->lang->line('sales_giftcard'))
			{
				$data['error'] = $this->lang->line('sales_must_enter_numeric_giftcard');
			}
			else
			{
				$data['error'] = $this->lang->line('sales_must_enter_numeric');
			}
		}
		else
		{
			if($payment_type == $this->lang->line('sales_giftcard'))
			{
				// in case of giftcard payment the register input amount_tendered becomes the giftcard number
				$giftcard_num = $this->input->post('amount_tendered');

				$payments = $this->sale_lib->get_payments();
				$payment_type = $payment_type . ':' . $giftcard_num;
				$current_payments_with_giftcard = isset($payments[$payment_type]) ? $payments[$payment_type]['payment_amount'] : 0;
				$cur_giftcard_value = $this->Giftcard->get_giftcard_value($giftcard_num);
				
				if(($cur_giftcard_value - $current_payments_with_giftcard) <= 0)
				{
					$data['error'] = $this->lang->line('giftcards_remaining_balance', $giftcard_num, to_currency($cur_giftcard_value));
				}
				else
				{
					$new_giftcard_value = $this->Giftcard->get_giftcard_value($giftcard_num) - $this->sale_lib->get_amount_due();
					$new_giftcard_value = $new_giftcard_value >= 0 ? $new_giftcard_value : 0;
					$this->sale_lib->set_giftcard_remainder($new_giftcard_value);
					$new_giftcard_value = str_replace('$', '\$', to_currency($new_giftcard_value));
					$data['warning'] = $this->lang->line('giftcards_remaining_balance', $giftcard_num, $new_giftcard_value);
					$amount_tendered = min( $this->sale_lib->get_amount_due(), $this->Giftcard->get_giftcard_value($giftcard_num) );

					$this->sale_lib->add_payment($payment_type, $amount_tendered);
				}
			}
			else
			{
				$amount_tendered = $this->input->post('amount_tendered');

				$this->sale_lib->add_payment($payment_type, $amount_tendered);
			}
		}

		$this->_reload($data);
	}

	// Multiple Payments
	public function delete_payment($payment_id)
	{
		$this->sale_lib->delete_payment($payment_id);

		$this->_reload();
	}

	public function add()
	{
		$data = array();

		$discount = 0;
		
		$item = '';
		$item_adicion = 0;
		
		//AGREGAR ARTICULO GENERICO CUANDO SE PRESIONA +
		$add_generic = $this->input->post('add_generic');
		if($add_generic =='1'){
			$last_generic = $this->sale_lib->get_last_generic_item();
			$len_last_generic = strlen($last_generic);
			if ($len_last_generic == 0){
				$this->sale_lib->set_last_generic_item(2000000000008);//EN DURO POR AHORA CODIGO GENERICO
			}
			else{
				$this->sale_lib->set_last_generic_item($last_generic+1);
			}
			$item = $this->sale_lib->get_last_generic_item();
			$price = $this->input->post('item');
		}else{

			//Verificar si se cargó con boton
			$boton = null;
			$boton = $this->input->get('boton');
			if ($boton == 1) {
				//echo "get";
				//INGRESA POR GET

				$item = $this->input->get('item');
				//echo "get ".$item;
				$item = $this->Item->get_item_number($item);
				//die();
			}else{
				//echo "post";
				//INGRESA POR POST
				$item = $this->input->post('item');
			}

			//revisar si se cargó con * asterisco 
			
			if (substr($item,0,1)=="*") {
//				echo "asterisco";
				$len_item = strlen($item);
				$item_adicion = 1;

				$item = substr($item,1,$len_item);
			
			}

		}
		
		// check if any discount is assigned to the selected customer
		$customer_id = $this->sale_lib->get_customer();//6107
		if($customer_id != -1)
		{
			// load the customer discount if any
			$discount_percent = $this->Customer->get_info($customer_id)->discount_percent;
			if($discount_percent != '')
			{
				$discount = $discount_percent;
			}
		}

		// if the customer discount is 0 or no customer is selected apply the default sales discount
		if($discount == 0)
		{
			$discount = $this->config->item('default_sales_discount');
		}



		$mode = $this->sale_lib->get_mode();
		$quantity = ($mode == 'return') ? -1 : 1;
		$item_location = $this->sale_lib->get_sale_location();

		//$item_id_or_number_or_item_kit_or_receipt = $this->input->post('item');

		$item_pre = substr($item,0,7);
		//echo " pre if ".$item_pre."<br>";
		$categoria=$this->Item->get_category($item_pre);
		//echo "Cate: ".$categoria;

		if ($categoria=='GRANEL'){
			$item_id_or_number_or_item_kit_or_receipt = $item_pre;
			$quantity = substr($item,7,2).".".substr($item,9,3);
			$price = NULL;
			//echo "cod: ".$item_id_or_number_or_item_kit_or_receipt." peso:".$quantity;
			//die();
		}elseif($categoria=='GRANEL NO PESABLE'){
			$item_id_or_number_or_item_kit_or_receipt = $item_pre;
			$price = substr($item,7,5);
			//echo "cod: ".$item_id_or_number_or_item_kit_or_receipt." peso:".$quantity;
			//die();
		}
		else
		{
		

			$item_id_or_number_or_item_kit_or_receipt = $item;
			//$item_id_or_number_or_item_kit_or_receipt = $item_id_get;
			if($add_generic != '1'){
				$price = NULL;	
			}
			
			//echo $item_id_or_number_or_item_kit_or_receipt;
		}

		$item_id_get=$this->Item->get_id_item($item_id_or_number_or_item_kit_or_receipt);
		$item_id_or_number_or_item_kit_or_receipt = $item_id_get;


		//Ver si se asignó la cantidad antes de leer el codigo
		if (strlen($this->input->post('cant'))>0){
			$quantity = $this->input->post('cant');
			
		}
		

		if($mode == 'return' && $this->Sale->is_valid_receipt($item_id_or_number_or_item_kit_or_receipt))
		{
			$this->sale_lib->return_entire_sale($item_id_or_number_or_item_kit_or_receipt);
		}
		elseif($this->Item_kit->is_valid_item_kit($item_id_or_number_or_item_kit_or_receipt))
		{
			if(!$this->sale_lib->add_item_kit($item_id_or_number_or_item_kit_or_receipt, $item_location, $discount))
			{
				$data['error'] = $this->lang->line('sales_unable_to_add_item');
			}
		}
		else
		{	$add_sales = 1;
			if(!$this->sale_lib->add_item($item_id_or_number_or_item_kit_or_receipt, $quantity, $item_location, $discount,$price,NULL,NULL, FALSE, 0, 0,$add_sales,$item_adicion))
			{

				//$data['error'] = $this->lang->line('sales_unable_to_add_item');
				$data['error'] = $this->sale_lib->get_mensaje_personalizado();
			}
			else
			{
				/*$data['warning'] = $this->sale_lib->out_of_stock($item_id_or_number_or_item_kit_or_receipt, $item_location);*/
				$data['warning'] = $this->sale_lib->out_of_stock($item_id_or_number_or_item_kit_or_receipt, $item_location);
			}
		}

		/*Borrar pagos en Efectivo cuando se agrega nuevos productos al carrito para poder recalcular*/
		$this->sale_lib->delete_payment('Efectivo');

		$this->_reload($data);
	}

	public function edit_item($item_id)
	{	
		$data = array();
		
		$this->form_validation->set_rules('price', 'lang:items_price', 'required|callback_numeric');
		$this->form_validation->set_rules('quantity', 'lang:items_quantity', 'required|callback_numeric');
		$this->form_validation->set_rules('discount', 'lang:items_discount', 'required|callback_numeric');

		$description = $this->input->post('description');
		$serialnumber = $this->input->post('serialnumber');
		$price = parse_decimals($this->input->post('price'));
		$quantity = parse_decimals($this->input->post('quantity'));
		$discount = 0;//parse_decimals($this->input->post('discount'));

		if (parse_decimals($this->input->post('discount'))<=parse_decimals($this->config->item('maximo_descuento'))) {
			$discount = parse_decimals($this->input->post('discount'));
		}
		else
		{
			$data['error'] = 'DESCUENTO MÁXIMO PERMITIDO: '.$this->config->item('maximo_descuento').' %';
		}	
		$item_location = $this->input->post('location');
		$boxes = $this->input->post('boxes');
		$pallets = $this->input->post('pallets');

		

		$item_number = $this->input->post('item_number');
		$item_adicion = 0;
			if (substr($item_number,0,1)=="*") {
							//echo "*";
				$item_adicion = 1;			
			}


		if($this->form_validation->run() != FALSE)
		{

			if(!$this->sale_lib->edit_item($item_id, $description, $serialnumber, $quantity, $discount, $price, $boxes,$pallets,$item_adicion))
			{

				//$data['error'] = $this->lang->line('sales_unable_to_add_item');
				$data['error'] = $this->sale_lib->get_mensaje_personalizado();
			}

			/*$this->sale_lib->edit_item($item_id, $description, $serialnumber, $quantity, $discount, $price, $boxes,$pallets);*/
		}
		else
		{
			$data['error'] = $this->lang->line('sales_error_editing_item');
		}

		$data['warning'] = $this->sale_lib->out_of_stock($this->sale_lib->get_item_id($item_id), $item_location);


		$this->_reload($data);
	}

	public function delete_item($item_number)
	{
		
		$this->sale_lib->delete_item($item_number);

		$this->_reload();
	}

	public function remove_customer()
	{
		$this->sale_lib->clear_giftcard_remainder();
		$this->sale_lib->clear_invoice_number();
		$this->sale_lib->remove_customer();

		$this->_reload();
	}


	public function remove_seller()
	{
		$this->sale_lib->remove_seller();
		$this->_reload();
	}

	public function complete()
	{
		$data = array();

		$data['cart'] = $this->sale_lib->get_cart();
		$cant_items = count($data['cart']);
		
		//solo para probar si guarda
		//$data['cart']=$data['cart1'];
		///////////////
		$data['seller'] = $this->sale_lib->get_seller();
		$data['subtotal'] = $this->sale_lib->get_subtotal();
		$data['discounted_subtotal'] = $this->sale_lib->get_subtotal(TRUE);
		$data['tax_exclusive_subtotal'] = $this->sale_lib->get_subtotal(TRUE, TRUE);
		$data['taxes'] = $this->sale_lib->get_taxes();
		$data['total'] = $this->sale_lib->get_total();
		$data['discount'] = $this->sale_lib->get_discount();
		$data['receipt_title'] = $this->lang->line('sales_receipt');
		$data['transaction_time'] = date($this->config->item('dateformat') . ' ' . $this->config->item('timeformat'));
		$data['transaction_date'] = date($this->config->item('dateformat'));
		$data['show_stock_locations'] = $this->Stock_location->show_locations('sales');
		$data['comments'] = $this->sale_lib->get_comment();
		$data['comments2'] = $this->sale_lib->get_comment2();
		$data['payments'] = $this->sale_lib->get_payments();
		$data['amount_change'] = $this->sale_lib->get_amount_due() * -1;
		$data['amount_due'] = $this->sale_lib->get_amount_due();
		$employee_id = $this->Employee->get_logged_in_employee_info()->person_id;
		$employee_info = $this->Employee->get_info($employee_id);
		$data['employee'] = $employee_info->first_name  . ' ' . $employee_info->last_name[0];
		$data['company_info'] = implode("\n", array(
			$this->config->item('address'),
			$this->config->item('phone'),
			$this->config->item('account_number')
		));


		$seller_info = $this->Employee->get_info($data['seller']);
		$data['seller_name'] = $seller_info->first_name  . ' ' . $seller_info->last_name[0];
		

		/*Guardar ultimo vuelto en la sesion*/
		$this->sale_lib->set_ultimo_vuelto($data['amount_due']);

		$customer_id = $this->sale_lib->get_customer();
		$customer_info = $this->_load_customer_data($customer_id, $data);

		//$invoice_number = $this->_substitute_invoice_number($customer_info);
		$invoice_number = $this->Sale->get_next_invoice_number();

		//Saltar numeros de factura cuando hay más de 10 items
		/*if($this->sale_lib->is_invoice_number_enabled())
		{
			$data['comments'] = $data['comments']."*".$invoice_number."*";
			if($cant_items > 23)
			{
				for($f=11;$f<=$cant_items;$f=$f+10){
					
					$invoice_number = $invoice_number + 1 ;
					//Agregar info de varias facturas en comentarios
					$data['comments'] = $data['comments']."*".$invoice_number."*";
				}
			}
		}*/


		$data['customer_name'] = $customer_info->first_name." ".$customer_info->last_name;
		$data['customer_ruc'] = $customer_info->address_2;
		$data['customer_address'] = $customer_info->address_1;
		$data['customer_phone_number'] = $customer_info->customer_phone_number;
		
		//echo $data['customer_name'].$data['customer_ruc'];
		//var_dump($data['cliente']);
		//die();

		//Sumar pagos
		
		//echo "Sub ".$data['subtotal']."<br>";
		//echo "desc ".$data['discount']."<br>";
		//echo "con desc " .$data['discounted_subtotal']."<br>";
		$pago = 0;
		foreach($data['payments'] as $payment_id=>$payment)
		{ 
		$pago = $pago + round($payment['payment_amount']);
		}
		//echo $pago;
		//Controlar que el importe pagado sea igual o mayor al subtotal
		$r = 0;

		$delivery = 0;
		foreach(array_reverse($data['cart'], true) as $line=>$item)
		{
			if ($item['quantity']==0) 
			{
				$data['error'] = 'Verifique cantidad de producto en cero';
				//$this->_reload($data);
				$r = 1;
			}
			/*if ($item['price']==0) 
			{
				$data['error'] = 'Verifique precio de producto en cero';
				//$this->_reload($data);
				$r = 1;
			}*/

			if ($item['category']=='DELIVERY')
			{
				$delivery = $delivery + 1;
			}
		}


		if ($data['comments']=='DELIVERY' && $delivery==0) 
		{
			$data['error'] = 'Debe cargar el DELIVERY';
				//$this->_reload($data);
			$r = 1;
		}


		if (round($data['discount'])>0) {
			if (round($data['discounted_subtotal'])>round($pago)) {
				$r= 1;
			}
		}
		else{
			if (round($data['subtotal'])>round($pago)) {
				$r= 1;
			}
		}
		//echo "r ".$r;
		//die();
		//Si el pago es menor al subtotal se recarga la pagina
		if($r == 1){

			//echo "pago ".round($data['discounted_subtotal'])." ".round($pago);
			$this->_reload($data);
		}
		else
		{

		if($this->sale_lib->is_invoice_number_enabled() && $this->Sale->check_invoice_number_exists($invoice_number))
		{
			$data['error'] = $this->lang->line('sales_invoice_number_duplicate');

			$this->_reload($data);
		}
		else 
		{

			
		
			$invoice_number = $this->sale_lib->is_invoice_number_enabled() ? $invoice_number : NULL;
			$data['invoice_number'] = $invoice_number;

			$seller_id = $data['seller'];

			$data['sale_id_num'] = $this->Sale->save($data['cart'], $customer_id, $seller_id, $employee_id, $data['comments'], $invoice_number, $data['payments'],$data['comments2']);

			//$data['sale_id_num'] = $this->Sale->save($cart1, $customer_id, $employee_id, $data['comments'], $invoice_number, $data['payments']);

			/*$sale_id_num = $this->Sale->save($cart2, $customer_id, $employee_id, $data['comments'], $invoice_number+1, $data['payments']);*/
		



			$data['sale_id'] = 'POS ' . $data['sale_id_num'];
			
			$data = $this->xss_clean($data); 
			
			if($data['sale_id_num'] == -1)
			{
				$data['error_message'] = $this->lang->line('sales_transaction_failed');
			}
			else
			{
				$data['barcode'] = $this->barcode_lib->generate_receipt_barcode($data['sale_id']);
			}

			$data['cur_giftcard_value'] = $this->sale_lib->get_giftcard_remainder();
			
			$data['print_after_sale'] = $this->sale_lib->is_print_after_sale();
			
			//Vuelve a poner imprimir NO
			/*if ($data['print_after_sale']=='1'){
				$this->sale_lib->unset_print_after_sale();
			}
			else
			{
				$data['print_after_sale'] = '0';
			}*/
			

			$data['email_receipt'] = $this->sale_lib->get_email_receipt();

			$data['factura'] = 0;
			
			//echo "pre";
			$id_venta = $data['sale_id_num'];
			if($this->sale_lib->is_invoice_number_enabled() )
			{
				//if ($data['print_after_sale']=='0') 
				//{
					//$this->load->view('sales/invoice', $data);
					
				//}	
					


					//////ENVIAR JSON////////
					// Realizar la consulta SQL
					$query = "SELECT s.invoice_number comprobante, date(s.sale_time) fecha, 'FACT' tipo, 'GS' moneda,
					            (SELECT SUM(si.quantity_purchased*(si.item_unit_price-(si.item_unit_price*si.discount_percent/100))) 
					            FROM ospos_sales_items si, ospos_items_taxes i
					            WHERE si.sale_id=s.sale_id AND si.item_id=i.item_id AND i.percent=10) gravadas10,
					            (SELECT SUM(si.quantity_purchased*(si.item_unit_price-(si.item_unit_price*si.discount_percent/100))) 
					            FROM ospos_sales_items si, ospos_items_taxes i
					            WHERE si.sale_id=s.sale_id AND si.item_id=i.item_id AND i.percent=5) gravadas5,
					            (SELECT SUM(si.quantity_purchased*(si.item_unit_price-(si.item_unit_price*si.discount_percent/100))) 
					            FROM ospos_sales_items si, ospos_items_taxes i
					            WHERE si.sale_id=s.sale_id AND si.item_id=i.item_id AND i.percent=0) exentas,
					            CONCAT(p.first_name,' ',p.last_name) cliente,
					            p.address_2 ruc,
					            s.sale_id
					            FROM ospos_sales s, ospos_people p
					            WHERE s.customer_id=p.person_id AND (LENGTH(s.invoice_number) > 0)
					            AND s.sale_id=?";

					$result = $this->db->query($query, array($id_venta))->result();
					$contrato = $this->config->item('webservice_contrato');
					$url = $this->config->item('webservice_url');
					$token = $this->config->item('webservice_token');
					$hoy = date("d-m-Y");
					// Transformar los resultados en el formato deseado
					$data2 = array(
					    "contrato" => $contrato,
					    "fecha" => $hoy,
					    "ventas" => array()
					);

					$comprobante = '';
					$fecha = '';
					$tipo = '';
					$moneda = '';
					$tipoCambio = '';
					$gravadas10 = '';
					$gravadas5 = '';
					$exentas = '';
					$total = '';
					$cliente = '';
					$ruc = '';

					foreach ($result as $row) {
						$comprobante= str_pad($row->comprobante, 13, "0", STR_PAD_LEFT);
                        $comprobante= str_replace('-', '', $comprobante);
					    $venta = array(
					        "comprobante" => substr($comprobante, 0,3).'-'.substr($comprobante, 3,3).'-'.substr($comprobante, 6),
					        "fecha" => date("d-m-o", strtotime($row->fecha)),
					        "tipo" => $row->tipo,
					        "moneda" => $row->moneda,
					        "tipoCambio" => "0",
					        "gravadas10" => number_format($row->gravadas10,2,".",""),
					        "gravadas5" => number_format($row->gravadas5,2,".",""),
					        "exentas" => number_format($row->exentas,2,".",""),
					        "total" => number_format($row->gravadas10 + $row->gravadas5 + $row->exentas,2,".",""),
					        "cliente" => $row->cliente,
					        "ruc" => $row->ruc
					    );

					    $comprobante = substr($comprobante, 0,3).'-'.substr($comprobante, 3,3).'-'.substr($comprobante, 6);
						$fecha = date("d-m-o", strtotime($row->fecha));
						$tipo = $row->tipo;
						$moneda = $row->moneda;
						$tipoCambio = "0";
						$gravadas10 = number_format($row->gravadas10,2,".","");
						$gravadas5 = number_format($row->gravadas5,2,".","");
						$exentas = number_format($row->exentas,2,".","");
						$total = number_format($row->gravadas10 + $row->gravadas5 + $row->exentas,2,".","");
						$cliente = $row->cliente;
						$ruc = $row->ruc;

					    $data2["ventas"][] = $venta;
					}

					///tercera version
					$curl = curl_init();

					$postfields = '{"contrato": "'.$contrato.'","fecha": "'.$hoy.'","ventas": [{"comprobante": "'.$comprobante.'","fecha": "'.$fecha.'","tipo": "'.$tipo.'","moneda": "'.$moneda.'","tipoCambio": "'.$tipoCambio.'","gravadas10": "'.$gravadas10.'","gravadas5": "'.$gravadas5.'","exentas": "'.$exentas.'","total": "'.$total.'","cliente": "'.$cliente.'","ruc": "'.$ruc.'"}]}';

					curl_setopt($curl, CURLOPT_URL, $url);
					curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
					curl_setopt($curl, CURLOPT_ENCODING, '');
					curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
					curl_setopt($curl, CURLOPT_TIMEOUT, 0);
					curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
					curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
					curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
					curl_setopt($curl, CURLOPT_POSTFIELDS, $postfields);
					curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
					//curl_setopt($curl, CURLOPT_CAINFO, './cacert.pem');
					curl_setopt($curl, CURLOPT_HTTPHEADER, array(
					    'Accept: application/json',
					    'Content-Type: application/json',
					    'Authorization: '.$token,
					));

					/*$response = curl_exec($curl);

					if ($response === false) {
					    $error_curl = curl_error($curl);
					    echo 'Error en cURL: ' . $error_curl;
					    $response=$response.' Error en cURL: '.$error_curl;
					} else {
					    echo $response;
					}

					curl_close($curl);

					$response = $comprobante.' '.$response."\r";
					$printer ="c:\\Sistema\\log.txt";
					$fp=fopen($printer, 'a+');
					fwrite($fp,$response);
					fclose($fp);


					
					$printer2 ="c:\\Sistema\\factura.txt";
					$fp=fopen($printer2, 'w');
					fwrite($fp,$postfields);
					fclose($fp);
					// ...
					*/
					// Cerrar la conexión cURL
					curl_close($curl);
					
					//////FIN ENVIAR JSON////
					$data['factura'] = 1;
					if ($data['print_after_sale']=='1' ) {
					//$this->sale_lib->unset_print_after_sale();
					$this->load->view('sales/receipt', $data);
					
					//$this->sale_lib->clear_all();
					//$this->receipt($id_venta );
					//echo "Espere";
					$this->sale_lib->unset_last_generic_item();


					$this->sale_lib->clear_all();

					$this->_reload();
				}else{
					
					//$this->invoice($data['sale_id_num']);
					$this->invoice($id_venta);
				}
			}
			else
			{
				$data['factura'] = 0;
				if ($data['print_after_sale']=='1' ) {
					//$this->sale_lib->unset_print_after_sale();
					
					
					//$this->sale_lib->clear_all();
					$this->receipt($id_venta );
					//$this->load->view('sales/receipt', $data);
					//echo "Espere";
					$this->sale_lib->unset_last_generic_item();


					$this->sale_lib->clear_all();

					$this->_reload();
				}else{
					
					$this->sale_lib->unset_last_generic_item();

					$this->sale_lib->clear_all();

					$this->_reload();
				}
			}

			$this->sale_lib->unset_last_generic_item();

			$this->sale_lib->clear_all();

		}
		}
	}

	public function send_invoice($sale_id)
	{
		$sale_data = $this->_load_sale_data($sale_id);
		$result = FALSE;
		$message = $this->lang->line('sales_invoice_no_email');

		if(!empty($sale_data['customer_email']))
		{
			$to = $sale_data['customer_email'];
			$subject = $this->lang->line('sales_invoice') . ' ' . $sale_data['invoice_number'];

			$text = $this->config->item('invoice_email_message');
			$text = str_replace('$INV', $sale_data['invoice_number'], $text);
			$text = str_replace('$CO', 'POS ' . $sale_data['sale_id'], $text);
			$text = $this->_substitute_customer($text, (object) $sale_data);

			// generate email attachment: invoice in pdf format
			$html = $this->load->view('sales/invoice_email', $sale_data, TRUE);
			// load pdf helper
			$this->load->helper(array('dompdf', 'file'));
			$filename = sys_get_temp_dir() . '/' . $this->lang->line('sales_invoice') . '-' . str_replace('/', '-' , $sale_data['invoice_number']) . '.pdf';
			if(file_put_contents($filename, pdf_create($html)) !== FALSE)
			{
				$result = $this->email_lib->sendEmail($to, $subject, $text, $filename);	
			}

			$message = $this->lang->line($result ? 'sales_invoice_sent' : 'sales_invoice_unsent') . ' ' . $to;
		}

		echo json_encode(array('success' => $result, 'message' => $message, 'id' => $sale_id));

		$this->sale_lib->clear_all();

		return $result;
	}

	public function send_receipt($sale_id)
	{
		$sale_data = $this->_load_sale_data($sale_id);

		$result = FALSE;
		$message = $this->lang->line('sales_receipt_no_email');

		if(!empty($sale_data['customer_email']))
		{
			$sale_data['barcode'] = $this->barcode_lib->generate_receipt_barcode($sale_data['sale_id']);

			$to = $sale_data['customer_email'];
			$subject = $this->lang->line('sales_receipt');

			$text = $this->load->view('sales/receipt_email', $sale_data, TRUE);
			
			$result = $this->email_lib->sendEmail($to, $subject, $text);

			$message = $this->lang->line($result ? 'sales_receipt_sent' : 'sales_receipt_unsent') . ' ' . $to;
		}

		echo json_encode(array('success' => $result, 'message' => $message, 'id' => $sale_id));

		$this->sale_lib->clear_all();

		return $result;
	}

	private function _substitute_variable($text, $variable, $object, $function)
	{
		// don't query if this variable isn't used
		if(strstr($text, $variable))
		{
			$value = call_user_func(array($object, $function));
			$text = str_replace($variable, $value, $text);
		}

		return $text;
	}

	private function _substitute_customer($text, $customer_info)
	{
		// substitute customer info
		$customer_id = $this->sale_lib->get_customer();
		if($customer_id != -1 && $customer_info != '')
		{
			$text = str_replace('$CU', $customer_info->first_name . ' ' . $customer_info->last_name, $text);
			$words = preg_split("/\s+/", trim($customer_info->first_name . ' ' . $customer_info->last_name));
			$acronym = '';
			foreach($words as $w)
			{
				$acronym .= $w[0];
			}
			$text = str_replace('$CI', $acronym, $text);
		}

		return $text;
	}

	private function _is_custom_invoice_number($customer_info)
	{
		$invoice_number = $this->config->config['sales_invoice_format'];
		$invoice_number = $this->_substitute_variables($invoice_number, $customer_info);

		return $this->sale_lib->get_invoice_number() != $invoice_number;
	}

	private function _substitute_variables($text, $customer_info)
	{
		$text = $this->_substitute_variable($text, '$YCO', $this->Sale, 'get_invoice_number_for_year');
		$text = $this->_substitute_variable($text, '$CO', $this->Sale , 'get_invoice_count');
		$text = $this->_substitute_variable($text, '$SCO', $this->Sale_suspended, 'get_invoice_count');
		$text = strftime($text);
		$text = $this->_substitute_customer($text, $customer_info);

		return $text;
	}

	private function _substitute_invoice_number($customer_info)
	{
		$invoice_number = $this->config->config['sales_invoice_format'];
		$invoice_number = $this->_substitute_variables($invoice_number, $customer_info);

		$this->sale_lib->set_invoice_number($invoice_number, TRUE);

		return $this->sale_lib->get_invoice_number();
	}

	private function _load_customer_data($customer_id, &$data, $totals = FALSE)
	{	
		$customer_info = '';

		if($customer_id != -1)
		{
			$customer_info = $this->Customer->get_info($customer_id);
			if(isset($customer_info->company_name))
			{
				$data['customer'] = $customer_info->company_name;
			}
			else
			{
				$data['customer'] = $customer_info->first_name . ' ' . $customer_info->last_name;
			}
			$data['customer_name'] = $customer_info->first_name . ' ' . $customer_info->last_name;
			$data['customer_ruc'] = $customer_info->address_2;
			$data['first_name'] = $customer_info->first_name;
			$data['last_name'] = $customer_info->last_name;
			$data['customer_email'] = $customer_info->email;
			$data['customer_address'] = $customer_info->address_1;
			$data['customer_phone_number'] = $customer_info->phone_number;
			if(!empty($customer_info->zip) or !empty($customer_info->city))
			{
				$data['customer_location'] = $customer_info->zip . ' ' . $customer_info->city;				
			}
			else
			{
				$data['customer_location'] = '';
			}
			$data['customer_account_number'] = $customer_info->account_number;
			$data['customer_discount_percent'] = $customer_info->discount_percent;
			if($totals)
			{
				$cust_totals = $this->Customer->get_totals($customer_id);

				$data['customer_total'] = $cust_totals->total;
			}
			$data['customer_info'] = implode("\n", array(
				$data['customer'],
				$data['customer_address'],
				$data['customer_location'],
				$data['customer_account_number']
			));
		}

		return $customer_info;
	}

	private function _load_sale_data($sale_id)
	{
		$this->sale_lib->clear_all();
		$sale_info = $this->Sale->get_info($sale_id)->row_array();
		$this->sale_lib->copy_entire_sale($sale_id);
		$data = array();
		$data['cart'] = $this->sale_lib->get_cart();
		//var_dump($data['cart']);
		//die();
		$data['seller'] = $this->sale_lib->get_seller();
		$data['payments'] = $this->sale_lib->get_payments();
		$data['subtotal'] = $this->sale_lib->get_subtotal();
		$data['discounted_subtotal'] = $this->sale_lib->get_subtotal(TRUE);
		$data['tax_exclusive_subtotal'] = $this->sale_lib->get_subtotal(TRUE, TRUE);
		$data['taxes'] = $this->sale_lib->get_taxes();
		$data['total'] = $this->sale_lib->get_total();
		$data['discount'] = $this->sale_lib->get_discount();
		$data['receipt_title'] = $this->lang->line('sales_receipt');
		$data['transaction_time'] = date($this->config->item('dateformat') . ' ' . $this->config->item('timeformat'), strtotime($sale_info['sale_time']));
		$data['transaction_date'] = date($this->config->item('dateformat'), strtotime($sale_info['sale_time']));
		$data['show_stock_locations'] = $this->Stock_location->show_locations('sales');
		$data['amount_change'] = $this->sale_lib->get_amount_due() * -1;
		$data['amount_due'] = $this->sale_lib->get_amount_due();
		$employee_info = $this->Employee->get_info($this->sale_lib->get_employee());
		$data['employee'] = $employee_info->first_name . ' ' . $employee_info->last_name[0];

		$seller_info = $this->Employee->get_info($data['seller']);
		$data['seller_name'] = $seller_info->first_name  . ' ' . $seller_info->last_name[0];
		
		$customer_info = $this->_load_customer_data($this->sale_lib->get_customer(), $data);
		
		
		$data['customer_name'] = $customer_info->first_name." ".$customer_info->last_name;
		$data['customer_ruc'] = $customer_info->address_2;
		$data['customer_ruc'] = $customer_info->address_2;
		$data['customer_address'] = $customer_info->address_1;
		$data['customer_phone_number'] = $customer_info->customer_phone_number;
		$data['customer_id'] = $customer_info->person_id;
		$data['sale_id_num'] = $sale_id;
		$data['factura'] = 0;
		$data['sale_id'] = 'POS ' . $sale_id;
		$data['comments'] = $sale_info['comment'];
		$data['comments2'] = $sale_info['comment2'];
		$data['invoice_number'] = $sale_info['invoice_number'];
		$data['company_info'] = implode("\n", array(
			$this->config->item('address'),
			$this->config->item('phone'),
			$this->config->item('account_number')
		));
		$data['barcode'] = $this->barcode_lib->generate_receipt_barcode($data['sale_id']);
		$data['print_after_sale'] = FALSE;
		$data['ticket_nro'] = $this->Sale->get_ticket($sale_id);
		return $this->xss_clean($data);
	}

	private function _reload($data = array())
	{		
		$data['turno'] = $this->sale_lib->get_turno();
		$data['seller'] = $this->sale_lib->get_seller();
		$data['ultimo_vuelto'] = $this->sale_lib->get_ultimo_vuelto();
		$data['cart'] = $this->sale_lib->get_cart();	 
		$data['modes'] = array('sale' => $this->lang->line('sales_sale'), 'return' => $this->lang->line('sales_return'));
		$data['mode'] = $this->sale_lib->get_mode();
		$data['stock_locations'] = $this->Stock_location->get_allowed_locations('sales');
		$data['stock_location'] = $this->sale_lib->get_sale_location();
		$data['subtotal'] = $this->sale_lib->get_subtotal(TRUE);
		$data['tax_exclusive_subtotal'] = $this->sale_lib->get_subtotal(TRUE, TRUE);
		$data['taxes'] = $this->sale_lib->get_taxes();
		$data['discount'] = $this->sale_lib->get_discount();
		$data['total'] = $this->sale_lib->get_total();
		$data['comment'] = $this->sale_lib->get_comment();
		$data['comment2'] = $this->sale_lib->get_comment2();
		$data['email_receipt'] = $this->sale_lib->get_email_receipt();
		$data['payments_total'] = $this->sale_lib->get_payments_total();
		$data['amount_due'] = $this->sale_lib->get_amount_due();
		$data['payments'] = $this->sale_lib->get_payments();
		$data['payment_options'] = $this->Sale->get_payment_options();
		
		$data['items_module_allowed'] = $this->Employee->has_grant('items', $this->Employee->get_logged_in_employee_info()->person_id);

		$customer_info = $this->_load_customer_data($this->sale_lib->get_customer(), $data, FALSE);
		$data['invoice_number'] = $this->Sale->get_next_invoice_number();//5555;//$this->_substitute_invoice_number($customer_info);

		$data['invoice_number_enabled'] = $this->sale_lib->is_invoice_number_enabled();
		$data['print_after_sale'] = $this->sale_lib->is_print_after_sale();
		$data['payments_cover_total'] = $this->sale_lib->get_amount_due() <= 0;

		$data['seller_name'] = $this->Person->get_name($this->sale_lib->get_seller());
		
		//Habilitación de caja

		$data['habilitacion_caja'] = $this->Sale->get_habilitacion_caja($this->Employee->get_logged_in_employee_info()->person_id);

		//$this->sale_lib->unset_print_after_sale();
		/*if ($data['invoice_number_enabled'] == 1){
			$this->sale_lib->unset_print_after_sale();	
		}
		else{
			$this->sale_lib->set_print_after_sale(1);	
		}*/
		$this->sale_lib->set_print_after_sale(1);
		$this->sale_lib->set_invoice_number_enabled(TRUE);
		/*Traer datos de botones POS*/
		$category_pos_init = $this->sale_lib->get_category_pos();
		$category_pos = $this->Item->get_pos_categories();
		$item_pos = $this->Item->get_pos_items_category($category_pos_init);
		//$item_pos = $this->Item->get_pos_items_order();
		$restaurant_table = $this->sale_lib->get_restaurant_table();
		$restaurant_table_data_row = $this->Sale->get_restaurant_table_data_row($restaurant_table);
		//echo "string 1";
		$mesas = $this->Item->get_mesas();
		//echo "string 2";
		$data = $this->xss_clean($data);


		
		//echo "string 2";
		//$data['item_pos'] = array();
		$data['category_pos_init'] = $category_pos_init;
		$data['category_pos'] = $category_pos;
		$data['item_pos'] = $item_pos;
		$data['restaurant_table'] = $restaurant_table;
		$data['restaurant_table_data_row'] = $restaurant_table_data_row;

		$items = $this->Sale->get_restaurant_tables();
	   	$data['tables']=$items;
		$data['mesas']=$mesas;
		//var_dump($data['item_pos']);

		$this->load->view("sales/register", $data);
	}

	public function receipt($sale_id)
	{
		$data = $this->_load_sale_data($sale_id);

		$this->load->view('sales/receipt', $data);

		$this->sale_lib->clear_all();
	}

	public function invoice($sale_id)
	{
		$data = $this->_load_sale_data($sale_id);

		$this->load->view('sales/invoice', $data);

		$this->sale_lib->clear_all();
	}

	public function edit($sale_id)
	{
		$data = array();

		$data['employees'] = array();
		foreach($this->Employee->get_all()->result() as $employee)
		{
			foreach(get_object_vars($employee) as $property => $value)
			{
				$employee->$property = $this->xss_clean($value);
			}
			
			$data['employees'][$employee->person_id] = $employee->first_name . ' ' . $employee->last_name;
		}

		$sale_info = $this->xss_clean($this->Sale->get_info($sale_id)->row_array());	
		$data['selected_customer_name'] = $sale_info['customer_name'];
		$data['selected_customer_id'] = $sale_info['customer_id'];
		$data['sale_info'] = $sale_info;

		$data['payments'] = array();
		foreach($this->Sale->get_sale_payments($sale_id)->result() as $payment)
		{
			foreach(get_object_vars($payment) as $property => $value)
			{
				$payment->$property = $this->xss_clean($value);
			}
			
			$data['payments'][] = $payment;
		}
		
		// don't allow gift card to be a payment option in a sale transaction edit because it's a complex change
		$data['payment_options'] = $this->xss_clean($this->Sale->get_payment_options(FALSE));
		
		$this->load->view('sales/form', $data);
	}

	public function delete($sale_id = -1, $update_inventory = TRUE)
	{
		$employee_id = $this->Employee->get_logged_in_employee_info()->person_id;
		$sale_ids = $sale_id == -1 ? $this->input->post('ids') : array($sale_id);

		if($this->Sale->delete_list($sale_ids, $employee_id, $update_inventory))
		{
			echo json_encode(array('success' => TRUE, 'message' => $this->lang->line('sales_successfully_deleted') . ' ' .
							count($sale_ids) . ' ' . $this->lang->line('sales_one_or_multiple'), 'ids' => $sale_ids));
		}
		else
		{
			echo json_encode(array('success' => FALSE, 'message' => $this->lang->line('sales_unsuccessfully_deleted')));
		}
	}

	public function save($sale_id = -1)
	{
		$newdate = $this->input->post('date');
		$date_formatter = date_create_from_format($this->config->item('dateformat') . ' ' . $this->config->item('timeformat'), $newdate);

		$sale_data = array(
			'sale_time' => $date_formatter->format('Y-m-d H:i:s'),
			'customer_id' => $this->input->post('customer_id') != '' ? $this->input->post('customer_id') : NULL,
			'employee_id' => $this->input->post('employee_id'),
			'comment' => $this->input->post('comment'),
			'invoice_number' => $this->input->post('invoice_number') != '' ? $this->input->post('invoice_number') : NULL
		);

		// go through all the payment type input from the form, make sure the form matches the name and iterator number
		$mensaje = '';
		$payments = array();
		$number_of_payments = $this->input->post('number_of_payments');
		$total_payments = 0;
		for ($i = 0; $i < $number_of_payments; ++$i)
		{
			//$mensaje = $payment_amount.' - ';
			$total_payments = $total_payments + $this->input->post('payment_amount_' . $i);
			$payment_amount = $this->input->post('payment_amount_' . $i);
			$payment_type = $this->input->post('payment_type_' . $i);
			// remove any 0 payment if by mistake any was introduced at sale time
			if($payment_amount != 0)
			{
				// search for any payment of the same type that was already added, if that's the case add up the new payment amount
				$key = FALSE;
				if(!empty($payments))
				{
					// search in the multi array the key of the entry containing the current payment_type
					// NOTE: in PHP5.5 the array_map could be replaced by an array_column
					$key = array_search($payment_type, array_map(function($v){return $v['payment_type'];}, $payments));
				}

				// if no previous payment is found add a new one
				if($key === FALSE)
				{
					$payments[] = array('payment_type' => $payment_type, 'payment_amount' => $payment_amount);
				}
				else
				{
					// add up the new payment amount to an existing payment type
					$payments[$key]['payment_amount'] += $payment_amount;
				}
			}
		}

		$sale_info = $this->Sale->get_info($sale_id)->row_array();
		
		if ($sale_info['amount_due']==$total_payments) 
		{
			if($this->Sale->update($sale_id, $sale_data, $payments))
			{
				echo json_encode(array('success' => TRUE, 'message' => $mensaje.$this->lang->line('sales_successfully_updated'), 'id' => $sale_id));
			}
			else
			{
				echo json_encode(array('success' => FALSE, 'message' => $this->lang->line('sales_unsuccessfully_updated'), 'id' => $sale_id));
			}
		}
		else
		{
			echo json_encode(array('success' => FALSE, 'message' => $this->lang->line('sales_unsuccessfully_updated').' IMPORTE DE PAGOS INCORRECTO '.$total_payments, 'id' => $sale_id));
		}
		
	}

	public function cancel()
	{
		$this->sale_lib->unset_last_generic_item();
		$this->sale_lib->clear_all();

		$this->_reload();
	}

	public function suspend()
	{	
		$cart = $this->sale_lib->get_cart();
		$payments = $this->sale_lib->get_payments();
		$employee_id = $this->Employee->get_logged_in_employee_info()->person_id;
		$customer_id = $this->sale_lib->get_customer();
		$customer_info = $this->Customer->get_info($customer_id);
		$invoice_number = $this->_is_custom_invoice_number($customer_info) ? $this->sale_lib->get_invoice_number() : NULL;
		/*$comment = $this->sale_lib->get_comment();*/
		/*INFO DE MESAS RESATAURANT TABLES*/
		/*$table_id = $this->sale_lib->get_restaurant_table();	
		$restaurant_table_data_row = $this->Sale->get_restaurant_table_data_row($table_id);
		foreach($restaurant_table_data_row as $table_data_row)
    	{
    	$comment = $table_data_row->description." ÁREA: ". $table_data_row->area;
    	}*/
		/**/
		$comment = $this->sale_lib->get_comment();
		$comment2 = $this->sale_lib->get_comment2();
		//SAVE sale to database
		$data = array();
		if($this->Sale_suspended->save($cart, $customer_id, $employee_id, $comment, $invoice_number, $payments,$table_id,$comment2) == '-1')
		{
			$data['error'] = $this->lang->line('sales_unsuccessfully_suspended_sale');
		}
		else
		{
			$data['success'] = $this->lang->line('sales_successfully_suspended_sale');
		}

		$this->sale_lib->clear_all();

		$this->_reload($data);
	}
	
	public function suspended()
	{	
		$data = array();
		$data['suspended_sales'] = $this->xss_clean($this->Sale_suspended->get_all()->result_array());

		$this->load->view('sales/suspended', $data);
	}
	

	public function prices()
	{	
		
		

		$data = array();

		//$items= $this->Item->get_all();

		//SELECT item_id,name,item_number,custom2 FROM ospos_items i

		/*$this->db->select('ospos_items.item_id');
		$this->db->select('ospos_items.name');
		$this->db->select('ospos_items.item_number');
		$this->db->select('ospos_items.custom2');
		$this->db->from('ospos_items');
		$this->db->where('ospos_items.deleted',0);
		$items= $this->db->get();*/

   		//$data['items']=$items;

		//$data['suspended_sales'] = $this->xss_clean($this->Sale_suspended->get_all()->result_array());

		$this->load->view('sales/prices', $data);
	}


	public function prices_detail()
	{	
		
		$item_number = $this->input->post('item');
		$item_id = $this->Item->get_id_item($item_number);
		$item_info = $this->Item->get_info_by_id_or_number_custom($item_id);
		$custom2 = $item_info->custom2;
		$custom4 = $item_info->custom4;
		$name = $item_info->name;

		$data = array();
		//$items= $this->Item->get_all();

		//SELECT item_id,name,item_number,custom2 FROM ospos_items i

		$this->db->select('ospos_price_list.quantity_purchased');
		$this->db->select('ospos_price_list.item_unit_price');
		$this->db->select('ospos_price_list.pay_percent');
		$this->db->from('ospos_price_list');
		$this->db->join('ospos_prices_items', 'ospos_price_list.price_id=ospos_prices_items.price_id');
		$this->db->where('ospos_prices_items.item_id',$item_id);
		$items= $this->db->get();

   		$data['items']=$items;
   		$data['item_id']=$item_id;
   		$data['item_number']=$item_number;
		$data['custom2']=$custom2;
		$data['custom4']=$custom4;
		$data['name']=$name;

		$this->load->view('sales/prices_detail', $data);
	}	

	public function prices_edit()
	{	
		
		$item_id = $_GET["item_id"];
		$item_number = $_GET["item_number"];
		$custom2 = $_GET["custom2"];
		$custom1 = $_GET["custom1"];
		$name = $_GET["name"];

		$data = array();

		$price_id = $custom1;
		$description = $name;

		$prices= $this->Item->get_price_list($price_id);
		$items=$this->Item->get_prices_items($price_id);
   		$data['price_id'] = $price_id;
   		$data['description'] = $description;
   		$data['prices']=$prices;
   		$data['items']=$items;
		$this->load->view('items/price_list', $data);
	}


	public function unsuspend()
	{
		$sale_id = $this->input->post('suspended_sale_id');

		$this->sale_lib->clear_all();
		$this->sale_lib->copy_entire_suspended_sale($sale_id);
		$this->Sale_suspended->delete($sale_id);

		$this->_reload();
	}
	
	public function check_invoice_number()
	{
		$sale_id = $this->input->post('sale_id');
		$invoice_number = $this->input->post('invoice_number');
		$exists = !empty($invoice_number) && $this->Sale->check_invoice_number_exists($invoice_number, $sale_id);

		echo 'true';//!$exists ? 'true' : 'false';
	}

	/*Para que cualquiera con acceso a ventas pueda hacer backup*/

	public function backup_db()
    {
    	/*$employee_id = $this->Employee->get_logged_in_employee_info()->person_id;
    	if($this->Employee->has_module_grant('config', $employee_id))
    	{*/
    		$this->load->dbutil();

    		$prefs = array(
				'format' => 'zip',
				'filename' => 'bodegawil1.sql'
    		);
    		 
    		$backup = $this->dbutil->backup($prefs);
    		 
			$file_name = 'doesa-' . date("Y-m-d-H-i-s") .'.zip';
    		$save = 'uploads/' . $file_name;
    		$this->load->helper('download');
    		while(ob_get_level())
			{
    			ob_end_clean();
    		}

    		force_download($file_name, $backup);
    	/*}
    	else 
    	{
    		redirect('no_access/config');
    	}*/
    }

  
public function getFileList($dir)
  {
    // array to hold return value
    $retval = array();

    // add trailing slash if missing
    if(substr($dir, -1) != "/") $dir .= "/";

    // open pointer to directory and read list of files
    $d = @dir($dir) or die("getFileList: Failed opening directory $dir for reading");
    while(false !== ($entry = $d->read())) {
      // skip hidden files
      if($entry[0] == ".") continue;
      if(is_dir("$dir$entry")) {
        $retval[] = array(
   		  "name" => "$dir$entry/",
          "type" => filetype("$dir$entry"),
          "size" => 0,
          "lastmod" => filemtime("$dir$entry")
        );
      } elseif(is_readable("$dir$entry")) {
        $retval[] = array(
          "name" => "$dir$entry",
          "type" => mime_content_type("$dir$entry"),
          "size" => filesize("$dir$entry"),
          "lastmod" => filemtime("$dir$entry")
        );
      }
    }
    $d->close();

    return $retval;
  }

  	public function exports()
	{	
		
		$data = array();

		$dirlist = $this->getFileList("{$_SERVER['DOCUMENT_ROOT']}/exports/");
		//$dirlist = $this->getFileList("/opt/lampp/htdocs/exports");
		// /opt/lampp/htdocs/exports
		/*echo "string";
		echo "<pre>";
		print_r($dirlist);
		echo "</pre>";*/

		$data['dirlist'] = $dirlist;

		$this->load->view('sales/exports', $data);
	}
	
	public function prices_category()
	{	

		$data = array();
		//$items= $this->Item->get_all();

		//SELECT item_id,name,item_number,custom2 FROM ospos_items i

		$this->db->select('distinct(ospos_items.category)');
		$this->db->from('ospos_items');
		$this->db->where('ospos_items.deleted',0);
		$items= $this->db->get();

   		$data['items']=$items;
   		//echo "string";
   		//die();
		$this->load->view('sales/prices_category', $data);
	}

	public function presupuesto()
	{
		$data = array();

		$data['cart'] = $this->sale_lib->get_cart();
		$cant_items = count($data['cart']);
		
		$data['seller'] = $this->sale_lib->get_seller();
		$data['subtotal'] = $this->sale_lib->get_subtotal();
		$data['discounted_subtotal'] = $this->sale_lib->get_subtotal(TRUE);
		$data['tax_exclusive_subtotal'] = $this->sale_lib->get_subtotal(TRUE, TRUE);
		$data['taxes'] = $this->sale_lib->get_taxes();
		$data['total'] = $this->sale_lib->get_total();
		$data['discount'] = $this->sale_lib->get_discount();
		$data['receipt_title'] = $this->lang->line('sales_receipt');
		$data['transaction_time'] = date($this->config->item('dateformat') . ' ' . $this->config->item('timeformat'));
		$data['transaction_date'] = date($this->config->item('dateformat'));
		$data['show_stock_locations'] = $this->Stock_location->show_locations('sales');
		$data['comments'] = $this->sale_lib->get_comment();
		$data['payments'] = $this->sale_lib->get_payments();
		$data['amount_change'] = $this->sale_lib->get_amount_due() * -1;
		$data['amount_due'] = $this->sale_lib->get_amount_due();
		$employee_id = $this->Employee->get_logged_in_employee_info()->person_id;
		$employee_info = $this->Employee->get_info($employee_id);
		$data['employee'] = $employee_info->first_name  . ' ' . $employee_info->last_name[0];
		$data['company_info'] = implode("\n", array(
			$this->config->item('address'),
			$this->config->item('phone'),
			$this->config->item('account_number')
		));


		$seller_info = $this->Employee->get_info($data['seller']);
		$data['seller_name'] = $seller_info->first_name  . ' ' . $seller_info->last_name[0];
		

		/*Guardar ultimo vuelto en la sesion*/
		$this->sale_lib->set_ultimo_vuelto($data['amount_due']);

		$customer_id = $this->sale_lib->get_customer();
		$customer_info = $this->_load_customer_data($customer_id, $data);

		//$invoice_number = $this->_substitute_invoice_number($customer_info);
		$invoice_number = $this->Sale->get_next_invoice_number();

		//Saltar numeros de factura cuando hay más de 10 items
		if($this->sale_lib->is_invoice_number_enabled())
		{
			$data['comments'] = $data['comments']."*".$invoice_number."*";
			if($cant_items > 23)
			{
				for($f=11;$f<=$cant_items;$f=$f+10){
					
					$invoice_number = $invoice_number + 1 ;
					//Agregar info de varias facturas en comentarios
					$data['comments'] = $data['comments']."*".$invoice_number."*";
				}
			}
		}


		$data['customer_name'] = $customer_info->first_name." ".$customer_info->last_name;
		$data['customer_ruc'] = $customer_info->address_2;
		$data['customer_address'] = $customer_info->address_1;
		//echo $data['customer_name'].$data['customer_ruc'];
		//var_dump($data['cliente']);
		//die();

		//Sumar pagos
		
		//echo "Sub ".$data['subtotal']."<br>";
		//echo "desc ".$data['discount']."<br>";
		//echo "con desc " .$data['discounted_subtotal']."<br>";
		$pago = 0;
		foreach($data['payments'] as $payment_id=>$payment)
		{ 
		$pago = $pago + round($payment['payment_amount']);
		}
		//echo $pago;
		//Controlar que el importe pagado sea igual o mayor al subtotal
		$r = 0;
		if (round($data['discount'])>0) {
			if (round($data['discounted_subtotal'])>round($pago)) {
				$r= 1;
			}
		}
		else{
			if (round($data['subtotal'])>round($pago)) {
				$r= 1;
			}
		}
		//echo "r ".$r;
		//die();
		//Si el pago es menor al subtotal se recarga la pagina
		if($r == 1){

			//echo "pago ".round($data['discounted_subtotal'])." ".round($pago);
			$this->_reload();
		}
		else
		{

		if($this->sale_lib->is_invoice_number_enabled() && $this->Sale->check_invoice_number_exists($invoice_number))
		{
			$data['error'] = $this->lang->line('sales_invoice_number_duplicate');

			$this->_reload($data);
		}
		else 
		{

			
		
			$invoice_number = $this->sale_lib->is_invoice_number_enabled() ? $invoice_number : NULL;
			$data['invoice_number'] = $invoice_number;

			$seller_id = $data['seller'];

			/*$data['sale_id_num'] = $this->Sale->save($data['cart'], $customer_id, $seller_id, $employee_id, $data['comments'], $invoice_number, $data['payments']);*/

			//$data['sale_id_num'] = $this->Sale->save($cart1, $customer_id, $employee_id, $data['comments'], $invoice_number, $data['payments']);

			/*$sale_id_num = $this->Sale->save($cart2, $customer_id, $employee_id, $data['comments'], $invoice_number+1, $data['payments']);*/
		



			$data['sale_id'] = 'PRESUPUESTO';
			
			$data = $this->xss_clean($data); 
			
			if($data['sale_id_num'] == -1)
			{
				$data['error_message'] = $this->lang->line('sales_transaction_failed');
			}
			else
			{
				$data['barcode'] = $this->barcode_lib->generate_receipt_barcode($data['sale_id']);
			}

			$data['cur_giftcard_value'] = $this->sale_lib->get_giftcard_remainder();
			
			$data['print_after_sale'] = $this->sale_lib->is_print_after_sale();
			
			//Vuelve a poner imprimir NO
			if ($data['print_after_sale']=='1'){
				$this->sale_lib->unset_print_after_sale();
			}
			else
			{
				$data['print_after_sale'] = '0';
			}
			

			$data['email_receipt'] = $this->sale_lib->get_email_receipt();
			
			//echo "pre";

			$this->sale_lib->unset_print_after_sale();
			$this->load->view('sales/receipt', $data);
					//echo "Espere";
			//$this->sale_lib->unset_last_generic_item();


			//$this->sale_lib->clear_all();
			$this->_reload();


			//$this->sale_lib->unset_last_generic_item();

			//$this->sale_lib->clear_all();

		}
		}
	}

	public function caja($caja_id=0,$employee_id=0)
	{	
		
		

		$person_data = array();
		
		$hoy = date("Y-m-d H:i:s"); 
		//Estirar siempre del usuario 6 San Cayetano
		//$person_id = 6;
		
		//echo " *1 ";
		if ($employee_id==0) 
		{
			$employee_id=$this->Employee->get_logged_in_employee_info()->person_id;
		}
		//echo " *2 ";
		if ($caja_id==0) 
		{
			//echo " caja_0 ";
			//Habilitación de caja
			$person_id = $this->Employee->get_logged_in_employee_info()->person_id;
			$habilitacion_caja = $this->Sale->get_habilitacion_caja($person_id);
			$data['habilitacion_caja'] = $habilitacion_caja;
		}
		else
		{
			//echo " caja_".$caja_id;
			$data['habilitacion_caja'] = $caja_id;
			$habilitacion_caja = $caja_id;
		}	
		//echo "ha ".$habilitacion_caja;
		$data['caja_chica'] = $this->Sale->get_caja_chica($habilitacion_caja);

		//echo " *3 ";
		
		$sale_caja_apertura_time = $this->Sale->get_caja_apertura($habilitacion_caja);
		
		$data['sale_caja_apertura_time'] = $sale_caja_apertura_time;
		$data['cobros_clientes'] = 0;//$this->Sale->get_cobros_clientes($person_id,$sale_caja_apertura_time);
		//echo "1ch ".$sale_caja_apertura_time;
		$data['sale_total'] = $this->Sale->get_sale_total($habilitacion_caja);
		//echo " 2 ";
		$data['receiving_total'] = $this->Receiving->get_receiving_total($habilitacion_caja);
		//echo " 3 ";
		$data['libreta_total'] = 0;//$this->Sale->get_libreta_total($person_id,$sale_caja_apertura_time);
		//echo "2ch ".$data['caja_chica'];
		//die();

		$data['cobranzas_tipo'] = array();
		$data['cobranzas_tipo'] = $this->Sale->get_tipo_cobranza_caja($habilitacion_caja);

		$data['descuentos'] = $this->Sale->get_descuentos_caja($habilitacion_caja);
		

		if (strlen($data['sale_caja_cierre_time'])>0) 
		{
			$data['abierto'] = 0;
		}
		else
		{
			$data['abierto'] = 1;
		}
		//echo " fin ";
		$this->load->view('sales/caja', $data);
	}

	public function habilitar_caja()
	{
		$person_id = $_GET["person_id"];
		$caja_chica = $_GET["caja_chica"];
		


		$data['person_id']=$person_id;
		$data['caja_chica']=$caja_chica;

		$item_data = array(
			'person_id' => $person_id,
			'caja_chica' => $caja_chica
		);

		$this->db->insert('ospos_sales_caja', $item_data);
		$this->_reload();

	}

	public function cerrar_caja()
	{

	
		$sale_caja_id = $_GET["habilitacion_caja"];
		$sale_total = $_GET["sale_total"];
		$caja_cierre = $_GET["caja_cierre"];

		$sale_caja_cierre_time = date('Y-m-d H:i:s');

		$caja_chica = $this->Sale->get_caja_chica($sale_caja_id);
		$sale_caja_apertura_time = $this->Sale->get_caja_apertura($sale_caja_id);
		
	
		$item_data = array(
			'sale_caja_cierre_time' => $sale_caja_cierre_time,
			'sale_total' => $sale_total,
			'caja_cierre' => $caja_cierre
		);
		$this->db->where('sale_caja_id', $sale_caja_id);
		$success2 = $this->db->update('ospos_sales_caja', $item_data);

   		$data['sale_caja_apertura_time'] = $sale_caja_apertura_time;
   		$data['sale_caja_cierre_time'] = $sale_caja_cierre_time;
   		$data['caja_chica'] = $caja_chica;
   		$data['sale_caja_id']=$sale_caja_id;
   		$data['sale_total']=$sale_total;
   		$data['caja_cierre']=$caja_cierre;
   		$this->_reload();
   		//$this->load->view('sales/caja_cierre', $data);
	}



	public function listar_caja()
	{	
		
		$person_id = $_GET["person_id"];
		$fecha_ini = $_GET["fecha_ini"];
		$fecha_fin = $_GET["fecha_fin"];
		//echo "pre ";
		$sales = $this->Sale->get_caja_person_list($person_id,$fecha_ini, $fecha_fin);
		//echo "pre ";
		$empleado = $this->Employee->get_logged_in_employee_info()->first_name." ".$this->Employee->get_logged_in_employee_info()->last_name;
		$data = array();
		$data['sales'] = array();
		$data['sales'] = $sales;
		$data['fecha_ini'] = $fecha_ini;
		$data['fecha_fin'] = $fecha_fin;
		$data['person_id'] = $person_id;
		$data['title'] = "LISTADO DE CAJAS";
		$data['subtitle'] = 'Desde '.$fecha_ini.' hasta '.$fecha_fin;
		$data['empleado'] = $empleado;
		$this->load->view('sales/caja_list', $data);
	}

	public function listar_ventas()
	{	
		
		$person_id = $_GET["person_id"];
		$fecha_ini = $_GET["fecha_ini"];
		$fecha_fin = $_GET["fecha_fin"];

		$sales = $this->Sale->get_sale_person_list($person_id,$fecha_ini, $fecha_fin);
		$empleado = $this->Employee->get_logged_in_employee_info()->first_name." ".$this->Employee->get_logged_in_employee_info()->last_name;
		$data = array();
		$data['sales'] = array();
		$data['sales'] = $sales;
		$data['fecha_ini'] = $fecha_ini;
		$data['fecha_fin'] = $fecha_fin;
		$data['title'] = "LISTADO DE VENTAS";
		$data['subtitle'] = 'Desde '.$fecha_ini.' hasta '.$fecha_fin;
		$data['empleado'] = $empleado;
		$this->load->view('sales/ventas_list', $data);
	}

	public function listar_ventas_det()
	{	
		
		$person_id = $_GET["person_id"];
		$fecha_ini = $_GET["fecha_ini"];
		$fecha_fin = $_GET["fecha_fin"];
		$sale_id = $_GET["sale_id"];

		$sales = $this->Sale->get_sale_list_det($sale_id);
		$empleado = $this->Employee->get_logged_in_employee_info()->first_name." ".$this->Employee->get_logged_in_employee_info()->last_name;
		$data = array();
		$data['sales'] = array();
		$data['sales'] = $sales;
		$data['fecha_ini'] = $fecha_ini;
		$data['fecha_fin'] = $fecha_fin;
		$data['title'] = "DETALLE DE VENTA";
		$data['subtitle'] = 'N° '.$sale_id;
		$data['empleado'] = $empleado;
		$this->load->view('sales/ventas_list_det', $data);
	}

	public function customer_touch()
	{	
		$customer_id = $this->input->post('customer_sn');
		$this->load->view('sales/customer_touch', $data);
	}
	
	public function customer_search()
	{

		$customer = $_GET["write"];

		$result = $this->Customer->search($customer);

		//var_dump($result);
		//die();

		$data['result'] = $result;
		$this->load->view('sales/customer_touch', $data);

		/*$customer_id = $this->Person->get_id($customer);
		
		if($this->Customer->exists($customer_id))
		{
			$this->sale_lib->set_customer($customer_id);

			$discount_percent = $this->Customer->get_info($customer_id)->discount_percent;

			// apply customer default discount to items that have 0 discount
			if($discount_percent != '')
			{	
				$this->sale_lib->apply_customer_discount($discount_percent);
			}
			$this->sale_lib->set_invoice_number_enabled(TRUE);

			$this->_reload();
			
		}else{
			
			$customer_id = $this->Person->get_id_from_name($customer);
			if($this->Customer->exists($customer_id))
			{
				$this->sale_lib->set_customer($customer_id);

				$discount_percent = $this->Customer->get_info($customer_id)->discount_percent;

				// apply customer default discount to items that have 0 discount
				if($discount_percent != '')
				{	
					$this->sale_lib->apply_customer_discount($discount_percent);
				}
				$this->sale_lib->set_invoice_number_enabled(TRUE);

				$this->_reload();
				
			}else{
				

				$data['error'] = "No se encontró";
				$this->load->view('sales/customer_touch', $data);
			}
			
		}*/
		
		
	}

	public function customer_select()
	{

		$customer_id = $this->input->get('customer_id');

		$this->sale_lib->set_customer($customer_id);

		$discount_percent = $this->Customer->get_info($customer_id)->discount_percent;
			// apply customer default discount to items that have 0 discount
		if($discount_percent != '')
			{	
				$this->sale_lib->apply_customer_discount($discount_percent);
			}
		$this->sale_lib->set_invoice_number_enabled(TRUE);

		$this->_reload();			
		
		
	}
	/*POS CATEGORIES*/
	public function set_pos_category()
	{	
		$category = $this->input->get('category');
		$this->sale_lib->set_category_pos($category);
		$this->_reload();
	}
	/*FIN POS CATEGORIES*/

	/*RESTAURANT TABLES*/
	public function restaurant_tables()
	{		
		
			$items = $this->Sale->get_restaurant_tables();
	   		$data['tables']=$items;
	   		$this->load->view('sales/restaurant_tables', $data);
	}

	public function set_restaurant_table()
	{	
		$table_id = $this->input->get('table_id');
		//echo "table: ".$table_id;
		//	die();
		$this->sale_lib->set_restaurant_table($table_id);
		$this->_reload();
	}

	/*FIN RESTAURANT TABLES*/

	public function turno()
	{	
		//echo "truno";
		$turno =$this->sale_lib->get_turno();
		
		$turno=$turno+1;
		if ($turno=='100') {
			$turno='00';
		}

		if (strlen($turno)==1) {
			$turno='0'.$turno;
		}
		$this->sale_lib->set_turno($turno);
		$this->sale_lib->set_comment($turno);

		$this->_reload();
	}

	public function comments()
	{	
		$comment = $this->input->get('commentario');
		$this->sale_lib->set_comment($comment);

		$this->_reload();
	}

	public function form_search_customer()
	{		
		$data['last_search'] = '';
		$this->load->view('sales/form_search_customer', $data);
	}

	public function search_customer()
	{	
		
		$search = $this->input->get('search_ruc');
		/*Buscar en la base de datos de RUC*/
		error_reporting(E_ALL ^ E_NOTICE);
		try
		{
			$bdd = new PDO('mysql:host=localhost:3306;dbname=ruc;charset=utf8', 'root', '');
		}
		catch(Exception $e)
		{
		        die('Error : '.$e->getMessage());
		}

		

		$sql = "SELECT ruc,razon_social,dv,alfanum FROM ruc WHERE ruc='".$search."'";

		$req = $bdd->prepare($sql);
		$req->execute();

		$rucs = $req->fetchAll();
		
		//var_dump($rucs);
		//die();
		
		$data['rucs'] = array();
		$data['rucs'] = $rucs;
		$data['last_search'] = $search;

		$this->load->view('sales/form_search_customer', $data);
	}

	public function save2($customer_id = -1)
	{
		$customer_id = $this->Customer->get_id($this->input->post('address_2'));
		
		if (is_null($customer_id)) {
			$customer_id = -1;
		}
		//echo "c ".$customer_id ;
		//die();
		$person_data = array(
			'first_name' => $this->input->post('first_name'),
			'last_name' => $this->input->post('last_name'),
			'gender' => $this->input->post('gender'),
			'email' => $this->input->post('email'),
			'phone_number' => $this->input->post('phone_number'),
			'address_1' => $this->input->post('address_1'),
			'address_2' => $this->input->post('address_2'),
			'city' => $this->input->post('city'),
			'state' => $this->input->post('state'),
			'zip' => $this->input->post('zip'),
			'country' => $this->input->post('country'),
			'comments' => $this->input->post('comments')
		);
		//echo "custo ";
		//echo $this->input->post('first_name');
		//die();
		$customer_data = array(
			'account_number' => $this->input->post('account_number') == '' ? NULL : $this->input->post('account_number'),
			'company_name' => $this->input->post('company_name') == '' ? NULL : $this->input->post('company_name'),
			'discount_percent' => $this->input->post('discount_percent') == '' ? 0.00 : $this->input->post('discount_percent'),
			'taxable' => $this->input->post('taxable') != NULL
		);

		if($this->Customer->save_customer($person_data, $customer_data, $customer_id))
		{
			$person_data = $this->xss_clean($person_data);
			$customer_data = $this->xss_clean($customer_data);
			
			//New customer
			if($customer_id == -1)
			{
				/*echo json_encode(array('success' => TRUE, 'message' => $this->lang->line('customers_successful_adding').' '.
								$person_data['first_name'].' '.$person_data['last_name'], 'id' => $customer_data['person_id']));*/
				$customer_id = $customer_data['person_id'];
				$data['success'] = 'Seleccionado'.' '.
								$person_data['first_name'].' '.$person_data['last_name'];

			}
			else //Existing customer
			{
				/*echo json_encode(array('success' => TRUE, 'message' => $this->lang->line('customers_successful_updating').' '.
								$person_data['first_name'].' '.$person_data['last_name'], 'id' => $customer_id));*/
				$data['success'] = $this->lang->line('customers_successful_updating').' '.
								$person_data['first_name'].' '.$person_data['last_name'];
			}
		}
		else//failure
		{
			$person_data = $this->xss_clean($person_data);

			/*echo json_encode(array('success' => FALSE, 'message' => $this->lang->line('customers_error_adding_updating').' '.
							$person_data['first_name'].' '.$person_data['last_name'], 'id' => -1));*/

			$data['error'] = $this->lang->line('customers_error_adding_updating').' '.
							$person_data['first_name'].' '.$person_data['last_name'];
		}
		
		$this->sale_lib->set_ruc($customer_id);
		if($this->Customer->exists($customer_id))
		{
			$this->sale_lib->set_customer($customer_id);

			$discount_percent = $this->Customer->get_info($customer_id)->discount_percent;

			// apply customer default discount to items that have 0 discount
			if($discount_percent != '')
			{	
				$this->sale_lib->apply_customer_discount($discount_percent);
			}
			$this->sale_lib->set_invoice_number_enabled(TRUE);
			
		}

		
		$this->_reload();
	}

	public function update_estado($sale_id,$estado,$redir='M')
	{
		//echo $sale_id.'-'.$estado;
		$estado_new = 'F';
		if ($estado=='F') 
		{
			$estado_new = 'I';
		}
		elseif ($estado=='I') 
		{
			$estado_new = 'T';
		}
		elseif ($estado=='T') 
		{
			$estado_new = 'D';
		}
		elseif ($estado=='D')
		{
			$estado_new = 'E';
		}
		elseif ($estado=='E')
		{
			$estado_new = 'F';
		}
		else
		{
			$estado_new = 'F';
		}
		
		$item_data = array(
			'estado' => $estado_new
		);
		$this->db->where('sale_id', $sale_id);
		$success2 = $this->db->update('ospos_sales', $item_data);
		if ($redir=='M') 
		{
			$this->manage();	
		}
		else
		{
			$this->cocina();	
		}
   		

	}

	public function cocina()
	{	
		$data = array();
		$data['habilitacion_caja'] = $this->Sale->get_habilitacion_caja($this->Employee->get_logged_in_employee_info()->person_id);
		$data['sales'] = $this->Sale->get_sale_estados($data['habilitacion_caja']);
		$this->load->view('sales/cocina', $data);
	}

	public function comments2()
	{	
		$comment2 = $this->input->get('commentario');
		$this->sale_lib->set_comment2($comment2);

		$this->_reload();
	}

	public function set_comment2() 
	{
		$this->sale_lib->set_comment2($this->input->post('comment2'));
	}

	public function detalle_ventas()
	{
		$data = array();

		
		//echo "string";
		
			$destino = $this->input->get('destino');
			$seller_id = $this->input->get('seller_id');
			$zona = $this->input->get('zona');
			$fecha_ini = $this->input->get('fecha_ini');
			$fecha_fin = $this->input->get('fecha_fin');
			$customer_id = $this->input->get('customer_id');
			$tipo = $this->input->get('tipo');
			$tipo_fecha = $this->input->get('tipo_fecha');
			$factura = $this->input->get('factura');
			$ticket = $this->input->get('ticket');

		if (!isset($destino)) {
			$destino = 'pantalla';
			//echo "p";
		}

		if (!isset($zona)) {
			$zona = 0;
			//echo "z";
		}
		if (strlen($zona) =='0') {
			$zona = 0;
			//echo "z";
		}

		if (!isset($seller_id)) {
			$seller_id = 0 ;
			//echo "s";
		}

		if (strlen($seller_id) =='0') {
			$seller_id = 0 ;
			//echo "s";
		}

		if (!isset($customer_id)) {
			$customer_id = 0 ;
			//echo "s";
		}

		if (strlen($customer_id) =='0') {
			$customer_id = 0 ;
			//echo "s";
		}

		if (!isset($factura)) {
			$factura = 0 ;
			//echo "s";
		}

		if (strlen($factura) =='0') {
			$factura = 0 ;
			//echo "s";
		}

		if (!isset($tipo_fecha)) {
			$tipo_fecha = 'fecha_pedido' ;
			//echo "s";
		}

		if (strlen($tipo_fecha) =='0') {
			$tipo_fecha = 'fecha_pedido' ;
			//echo "s";
		}

		if (strlen($fecha_ini) =='0') {
			$fecha_ini = date("Y-m-d H:i:s");
			
		}

		if (strlen($fecha_fin) =='0') {
			$fecha_fin = date("Y-m-d H:i:s");
			
		}

		if (strlen($ticket) =='0') {
			$ticket = 0 ;
			//echo "s";
		}

		if (strlen($tipo) =='0') {
			$tipo = 'ventas' ;
			//echo "s";
		}	

		//echo " pre ";
		if ($tipo=='ventas') 
		{
			//echo "ventas";
			$detalle_ventas= $this->Sale->get_detalle_ventas($zona,$seller_id,$fecha_ini,$fecha_fin,$customer_id,$factura,$tipo,$ticket);
			$data['title'] = 'DETALLE DE VENTAS';
			//echo " pos ven ";
		}
		if ($tipo=='ncr') 
		{
			//echo "ventas";
			$detalle_ventas= $this->Sale->get_detalle_ventas($zona,$seller_id,$fecha_ini,$fecha_fin,$customer_id,$factura,$tipo);
			$data['title'] = 'DETALLE DE NOTAS DE CREDITO';
		}
		if ($tipo=='pedidos') 
		{
			//echo " ped ";
			$detalle_ventas= $this->Sale->get_detalle_pedidos($zona,$seller_id,$fecha_ini,$fecha_fin,$customer_id,$tipo_fecha);
			$data['title'] = 'DETALLE DE PEDIDOS';
			//echo " pos ped ";
		}
		if ($tipo=='anuladas') {

			//echo " anul ";
			$detalle_ventas= $this->Sale->get_detalle_ventas_anuladas($zona,$seller_id,$fecha_ini,$fecha_fin,$customer_id);
			$data['title'] = 'DETALLE DE DOCUMENTOS ANULADOS';
		}
		//echo "hhh";
		$vendedor = $this->Employee->get_info($seller_id)->first_name.' '.$this->Employee->get_info($seller_id)->last_name;
		$data['detalle_ventas'] = array();
		$data['detalle_ventas'] = $detalle_ventas;

		$empleados = $this->Employee->get_empleados(0 );
		$data['empleados'] = array();
		$data['empleados'] = $empleados;

		$data['destino'] = $destino;
		$data['zona'] = $zona;
		$data['tipo'] = $tipo;
		$data['tipo_fecha'] = $tipo_fecha;
		$data['seller_id'] = $seller_id;
		$data['fecha_ini'] = $fecha_ini;
		$data['fecha_fin'] = $fecha_fin;
		$data['ticket'] = $ticket;
		$data['subtitle'] = 'Zona:'.$zona.' Vendedor:'.$vendedor.'('.$seller_id.') Fecha:'.$fecha_ini.' al '.$fecha_fin;
		$this->load->view('sales/detalle_ventas', $data);
	}

public function select_customer_detalle_ventas()
	{
		$customer_id = $this->input->post('customer');
		$destino = $this->input->post('destino');
		$seller_id = $this->input->post('seller_id');
		$zona = $this->input->post('zona');
		$fecha_ini = $this->input->post('fecha_ini');
		$fecha_fin = $this->input->post('fecha_fin');
		$cliente = $this->Customer->get_info($customer_id)->first_name.' '.$this->Customer->get_info($customer_id)->last_name;
		$data['customer_id'] = $customer_id;
		$data['cliente'] = $cliente;
		$data['destino'] = $destino;
		$data['zona'] = $zona;
		$data['seller_id'] = $seller_id;
		$data['fecha_ini'] = $fecha_ini;
		$data['fecha_fin'] = $fecha_fin;
		$data['title'] = 'DETALLE DE VENTAS POR CLIENTE';
		$data['subtitle'] = 'Zona:'.$zona.' Vendedor:'.$vendedor.'('.$seller_id.') Fecha:'.$fecha_ini.' al '.$fecha_fin;
		$this->load->view('sales/detalle_ventas', $data);
	}

	public function motivo_delete($sale_id,$motivo_id,$comentario)
	{
		$sale_id = $this->input->get('sale_id');
		$motivo_id = $this->input->get('motivo_id');
		$comentario = $this->input->get('comentario');
		$employee_id = $this->Employee->get_logged_in_employee_info()->person_id;
		//echo "string";
		$cabecera= $this->Sale->get_sale($sale_id);
		//var_dump($cabecera);
		$hoy = date("Y-m-d H:i:s"); 
		foreach($cabecera as $cab)
        {
           	$item_data = array(
			'sale_time' => $cab->sale_time,
			'customer_id' => $cab->customer_id,
			'employee_id' => $cab->employee_id,
			'comment' => $cab->comment,
			'invoice_number' => $cab->invoice_number,
			'sale_id' => $cab->sale_id,
			'seller_id' => $cab->seller_id,
			'mode' => $cab->mode,
			'caja_id' => $cab->caja_id,
			'pedido_id' => $cab->pedido_id,
			'delete_time' => $hoy,
			'motivo_id' => $motivo_id,
			'motivo_comment' => $comentario
			);
			$this->db->insert('ospos_sales_deleted', $item_data);
        }

        $detalle= $this->Sale->get_sale_items_id($sale_id);
        foreach($detalle as $cab)
        {
           	$item_datad = array(
			'sale_id' => $cab->sale_id,
			'item_id' => $cab->item_id,
			'description' => $cab->description,
			'serialnumber' => $cab->serialnumber,
			'line' => $cab->line,
			'quantity_purchased' => $cab->quantity_purchased,
			'item_cost_price' => $cab->item_cost_price,
			'item_unit_price' => $cab->item_unit_price,
			'discount_percent' => $cab->discount_percent,
			'item_location' => $cab->item_location
			);
			$this->db->insert('ospos_sales_deleted_items', $item_datad);
        }
        $this->Sale->delete($sale_id, $employee_id, TRUE);
		$this->_reload();
		

	}
}
?>