<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Sale_lib
{
	private $CI;

  	public function __construct()
	{
		$this->CI =& get_instance();
	}

	public function get_cart()
	{
		if(!$this->CI->session->userdata('sales_cart'))
		{
			$this->set_cart(array());
		}

		return $this->CI->session->userdata('sales_cart');
	}

	public function set_cart($cart_data)
	{
		$this->CI->session->set_userdata('sales_cart', $cart_data);
	}

	public function empty_cart()
	{
		$this->CI->session->unset_userdata('sales_cart');
	}
	
	public function set_ultimo_vuelto($ultimo_vuelto)
	{
		$this->CI->session->set_userdata('ultimo_vuelto', $ultimo_vuelto);
	}

	public function get_ultimo_vuelto()
	{
		$ultimo_vuelto = $this->CI->session->userdata('ultimo_vuelto');

    	return empty($ultimo_vuelto) ? '0' : $ultimo_vuelto;
	}

	public function get_comment() 
	{
		// avoid returning a NULL that results in a 0 in the comment if nothing is set/available
		$comment = $this->CI->session->userdata('sales_comment');

    	return empty($comment) ? '' : $comment;
	}

	public function set_comment($comment) 
	{
		$this->CI->session->set_userdata('sales_comment', $comment);
	}

	public function clear_comment() 	
	{
		$this->CI->session->unset_userdata('sales_comment');
	}
	
	public function get_invoice_number()
	{
		return $this->CI->session->userdata('sales_invoice_number');
	}
	
	public function set_invoice_number($invoice_number, $keep_custom = FALSE)
	{
		$current_invoice_number = $this->CI->session->userdata('sales_invoice_number');
		if(!$keep_custom || empty($current_invoice_number))
		{
			$this->CI->session->set_userdata('sales_invoice_number', $invoice_number);
		}
	}
	
	public function clear_invoice_number()
	{
		$this->CI->session->unset_userdata('sales_invoice_number');
	}
	
	public function is_invoice_number_enabled() 
	{
		return ($this->CI->session->userdata('sales_invoice_number_enabled') == 'true' ||
				$this->CI->session->userdata('sales_invoice_number_enabled') == '1') &&
				$this->CI->config->item('invoice_enable') == TRUE;
	}
	
	public function set_invoice_number_enabled($invoice_number_enabled)
	{
		return $this->CI->session->set_userdata('sales_invoice_number_enabled', $invoice_number_enabled);
	}
	
	public function is_print_after_sale() 
	{
		return ($this->CI->session->userdata('sales_print_after_sale') == 'true' ||
				$this->CI->session->userdata('sales_print_after_sale') == '1');
	}
	
	public function set_print_after_sale($print_after_sale)
	{
		return $this->CI->session->set_userdata('sales_print_after_sale', $print_after_sale);
	}

	public function unset_print_after_sale()
	{
		return $this->CI->session->unset_userdata('sales_print_after_sale');
	}
	

	public function set_last_generic_item($last_generic_item)
	{
		return $this->CI->session->set_userdata('last_generic_item', $last_generic_item);
	}

	public function get_last_generic_item()
	{
		return $this->CI->session->userdata('last_generic_item');
	}

	public function unset_last_generic_item()
	{
		return $this->CI->session->unset_userdata('last_generic_item');
	}

	public function get_email_receipt() 
	{
		return $this->CI->session->userdata('sales_email_receipt');
	}

	public function set_email_receipt($email_receipt) 
	{
		$this->CI->session->set_userdata('sales_email_receipt', $email_receipt);
	}

	public function clear_email_receipt() 	
	{
		$this->CI->session->unset_userdata('sales_email_receipt');
	}

	public function get_mensaje_personalizado()
	{
		return $this->CI->session->userdata('mensaje_personalizado');
	}

	public function set_ruc($customer_id) 
	{
		$this->CI->session->set_userdata('customer_ruc_input', $customer_id);
	}

	public function get_ruc() 
	{
		return $this->CI->session->userdata('customer_ruc_input');
	}
	// Multiple Payments
	public function get_payments()
	{
		if(!$this->CI->session->userdata('sales_payments'))
		{
			$this->set_payments(array());
		}

		return $this->CI->session->userdata('sales_payments');
	}

	// Multiple Payments
	public function set_payments($payments_data)
	{
		$this->CI->session->set_userdata('sales_payments', $payments_data);
	}

	// Multiple Payments
	public function add_payment($payment_id, $payment_amount)
	{
		$payments = $this->get_payments();
		if(isset($payments[$payment_id]))
		{
			//payment_method already exists, add to payment_amount
			$payments[$payment_id]['payment_amount'] = bcadd($payments[$payment_id]['payment_amount'], $payment_amount);
		}
		else
		{
			//add to existing array
			$payment = array($payment_id => array('payment_type' => $payment_id, 'payment_amount' => $payment_amount));
			
			$payments += $payment;
		}

		$this->set_payments($payments);
	}

	// Multiple Payments
	public function edit_payment($payment_id, $payment_amount)
	{
		$payments = $this->get_payments();
		if(isset($payments[$payment_id]))
		{
			$payments[$payment_id]['payment_type'] = $payment_id;
			$payments[$payment_id]['payment_amount'] = $payment_amount;
			$this->set_payments($payments);

			return TRUE;
		}

		return FALSE;
	}

	// Multiple Payments
	public function delete_payment($payment_id)
	{
		$payments = $this->get_payments();
		unset($payments[urldecode($payment_id)]);
		$this->set_payments($payments);
	}

	// Multiple Payments
	public function empty_payments()
	{
		$this->CI->session->unset_userdata('sales_payments');
	}

	// Multiple Payments
	public function get_payments_total()
	{
		$subtotal = 0;
		foreach($this->get_payments() as $payments)
		{
		    $subtotal = bcadd($payments['payment_amount'], $subtotal);
		}

		return $subtotal;
	}

	// Multiple Payments
	public function get_amount_due()
	{
		$payment_total = $this->get_payments_total();
		$sales_total = $this->get_total();
		$amount_due = bcsub($sales_total, $payment_total);
		$precision = $this->CI->config->item('currency_decimals');
		$rounded_due = bccomp(round($amount_due, $precision, PHP_ROUND_HALF_EVEN), 0, $precision);
		// take care of rounding error introduced by round tripping payment amount to the browser
 		return  $rounded_due == 0 ? 0 : $amount_due;
	}

	public function get_customer()
	{
		if(!$this->CI->session->userdata('sales_customer'))
		{
			$this->set_customer(5);
		}

		return $this->CI->session->userdata('sales_customer');
	}

	public function set_customer($customer_id)
	{
		$this->CI->session->set_userdata('sales_customer', $customer_id);
	}

	public function remove_customer()
	{
		$this->CI->session->unset_userdata('sales_customer');
	}
	
	public function get_seller()
	{
		if(!$this->CI->session->userdata('sales_seller'))
		{
			$this->set_seller(-1);
		}

		return $this->CI->session->userdata('sales_seller');
	}

	public function set_seller($employee_id)
	{
		$this->CI->session->set_userdata('sales_seller', $employee_id);
	}

	public function remove_seller()
	{
		$this->CI->session->unset_userdata('sales_seller');
	}

	public function get_employee()
	{
		if(!$this->CI->session->userdata('sales_employee'))
		{
			$this->set_employee(-1);
		}

		return $this->CI->session->userdata('sales_employee');
	}

	public function set_employee($employee_id)
	{
		$this->CI->session->set_userdata('sales_employee', $employee_id);
	}

	public function remove_employee()
	{
		$this->CI->session->unset_userdata('sales_employee');
	}

	public function get_mode()
	{
		if(!$this->CI->session->userdata('sales_mode'))
		{
			$this->set_mode('sale');
		}

		return $this->CI->session->userdata('sales_mode');
	}

	public function set_mode($mode)
	{
		$this->CI->session->set_userdata('sales_mode', $mode);
	}

	public function clear_mode()
	{
		$this->CI->session->unset_userdata('sales_mode');
	}

    public function get_sale_location()
    {
        if(!$this->CI->session->userdata('sales_location'))
        {
			$this->set_sale_location($this->CI->Stock_location->get_default_location_id());
        }

        return $this->CI->session->userdata('sales_location');
    }

    public function set_sale_location($location)
    {
        $this->CI->session->set_userdata('sales_location', $location);
    }
    
    public function clear_sale_location()
    {
    	$this->CI->session->unset_userdata('sales_location');
    }
    
    public function set_giftcard_remainder($value)
    {
    	$this->CI->session->set_userdata('sales_giftcard_remainder', $value);
    }
    
    public function get_giftcard_remainder()
    {
    	return $this->CI->session->userdata('sales_giftcard_remainder');
    }
    
    public function clear_giftcard_remainder()
    {
    	$this->CI->session->unset_userdata('sales_giftcard_remainder');
    }

	public function add_item(&$item_id, $quantity = 1, $item_location, $discount = 0, $price = NULL, $description = NULL, $serialnumber = NULL, $include_deleted = FALSE, $boxes = 0, $pallets = 0,$add_sales = 0,$item_adicion = 0)
	{
		
		$item_info = $this->CI->Item->get_info_by_id_or_number_custom($item_id);
		
		//make sure item exists		
		if(empty($item_info))
		{
			$item_id = -1;
            return FALSE;			
		}
		
		$item_id = $item_info->item_id;

		//Traer la lista de precios
		$price_id = $item_info->custom1;
		$item_box = $item_info->custom2;
		$item_pallet = $item_info->custom4;
		$price_quantity = 0;
		$quantity_adicion = $quantity;
		$pricekeys = array();
		$price_p = $price;
		//
		/*PRECIO DESDE FICHA*/
		$precio_ficha = $item_info->unit_price;
        
        if ($price_p!=0) 
        {
        	$price=$price_p;
        }
        
		// Serialization and Description

		//Get all items in the cart so far...
		$items = $this->get_cart();

        //We need to loop through all items in the cart.
        //If the item is already there, get it's key($updatekey).
        //We also need to get the next key that we are going to use in case we need to add the
        //item to the cart. Since items can be deleted, we can't use a count. we use the highest key + 1.

        $maxkey = 0;                       //Highest key so far
        $itemalreadyinsale = FALSE;        //We did not find the item yet.
		$insertkey = 0;                    //Key to use for new entry.
		$updatekey = 0;                    //Key to use to update(quantity)
		$cant_items = 0;

		foreach($items as $item)
		{
            //We primed the loop so maxkey is 0 the first time.
            //Also, we have stored the key in the element itself so we can compare.

			if($maxkey <= $item['line'])
			{
				$maxkey = $item['line'];
			}

			if($item['item_id'] == $item_id && $item['item_location'] == $item_location)
			{
				$itemalreadyinsale = TRUE;
				$updatekey = $item['line'];
                if(!$item_info->is_serialized)
                {
                    $quantity = bcadd($quantity, $items[$updatekey]['quantity']);
                }
			}

			//Cargar en un array los que tienen lista de precio igual y guardar las cantidades
			$price_item=$this->CI->Item->get_price_id($item['item_id']);
			//echo $price_item."<br>";
			//echo $item['name']." precio ".$price_item."<br>";
			
			if ($price_item == $price_id )
			{
				
				array_push($pricekeys, $item['line']);	
				if ($item['item_id'] != $item_id )
				{
					$price_quantity = bcadd($price_quantity, $items[$item['line']]['quantity']);	
				}	
				
			}
			$cant_items = $cant_items + 1;

		}

		//echo "cantidad con misma lista de precios ".$price_quantity;
		//die();

		$insertkey = $maxkey+1;
		//array/cart records are identified by $insertkey and item_id is just another field.
		
		//calcular precio segun cantidad y lista de precios

		//Sumar quantiy de lo que esta

		//echo $price_quantity." cant ".$quantity."<br>";
		$price_temp = $this->CI->Item->price_calc($item_id, $price_quantity+$quantity);
		$price_unit = $this->CI->Item->price_calc($item_id, 1);
		//echo $price_temp;
		//die();
		$price = $price != NULL ? $price : $price_temp;	

		//Precio de minorista para ventas con * asterisco, o sea, agregados a
        $item_number =$item_info->item_number;//
        if ($item_adicion == 1) {
        	$price = $price_unit;
        	$quantity = $quantity_adicion;
        	$itemalreadyinsale = FALSE;
        	$item_number = "*".$item_info->item_number;
        }

		//Precio de costo para cliente 6107 Bodega Wil 2
        /*$customer_id = $this->get_customer();//6107
        if ($customer_id == 61079999999) {
        	$price_unit = $item_info->cost_price;
        	$price = $item_info->cost_price;
        }*/

		//$price = $price != NULL ? $price : $item_info->unit_price;	
		//echo $quantity." * ".$price." = ".bcmul($quantity, $price);

        /*PRECIO DESDE FICHA*/
        $price = $precio_ficha;
        
        if ($price_p!=0) 
        {
        	$price=$price_p;
        }
        
		$total = $this->get_item_total($quantity, $price, $discount);
        $discounted_total = $this->get_item_total($quantity, $price, $discount, TRUE);


		
		//Item already exists and is not serialized, add to quantity
		if(!$itemalreadyinsale || $item_info->is_serialized)
		{
            $item = array($insertkey => array(
                    'item_id' => $item_id,
                    'item_location' => $item_location,
                    'stock_name' => $this->CI->Stock_location->get_location_name($item_location),
                    'line' => $insertkey,
                    'name' => $item_info->name,
                    'item_number' => $item_number,//$item_info->item_number,
                    'description' => $description != NULL ? $description : ''/*$item_info->description*/,
                    'serialnumber' => $serialnumber != NULL ? $serialnumber : '',
                    'allow_alt_description' => $item_info->allow_alt_description,
                    'is_serialized' => $item_info->is_serialized,
                    'quantity' => $quantity,
                    'discount' => $discount,
                    'pallets' => $pallets,
                    'boxes' => $boxes,
                    'category' => $item_info->category,
                    'item_box' => $item_box,
                    'item_pallet' => $item_pallet,
                    'tax' => $this->CI->Item->get_item_taxes($item_id),
                    'price_unit' => $price_unit,
                    'in_stock' => $this->CI->Item_quantity->get_item_quantity($item_id, $item_location)->quantity,
                    'price' => $price,
                    'total' => round($total),//para redondear por item
                    'discounted_total' => round($discounted_total),//para redondear por item
                    'category' => $item_info->category
                )
            );
			//add to existing array
			$items += $item;
		}
        else
        {
            $line = &$items[$updatekey];
            //para modificar el precio segun cantidad
            $line['price'] = $price;

            $line['quantity'] = $quantity;
            $line['total'] = $total;
            $line['discounted_total'] = $discounted_total;
        }

        //recorrer los items con la misma lista de precios y modificar el precio segun lo calculado
		/*PRECIO DESDE FICHA NO RECORRER*/
		/*foreach ($pricekeys as $pricekey) 
		{
    		$linea = &$items[$pricekey];
            //para modificar el precio segun cantidad o no modificar si es item adicional
             if ($item_adicion == 0) {
        		$linea['price'] = $price;
				$linea['total'] = $this->get_item_total($linea['quantity'], $price, $linea['discount']);
				$linea['discounted_total'] = $this->get_item_total($linea['quantity'], $price, $linea['discount'], TRUE);
        	}
            
		}*/
		
			if ($cant_items>36) {
				$mensaje = "No se puede superar el límite de artículos. Favor imprima la factura";
				$this->CI->session->set_userdata('mensaje_personalizado', $mensaje);
				return FALSE;

			}else{

				//echo "string ".$item_id." ".$item_location;
				if ($this->out_of_stock_lock($item_id, $item_location, $quantity)=='1') {
					# code...
					//echo "stock";
					$mensaje = "No se puede agregar el artículo a la venta";
					$this->CI->session->set_userdata('mensaje_personalizado', $mensaje);
					$this->set_cart($items);
					}

				else{
					//echo "no stock";
					$mensaje = "La cantidad deseada es insuficiente. Por favor verifica el inventario";
					$this->CI->session->set_userdata('mensaje_personalizado', $mensaje);
					if ($add_sales == 1) {
						//echo "no stock 2";
						//return FALSE; //Habilitar al tener stock
						$this->set_cart($items);//Deshabilitar al tener stock
					} else {//echo "no stock 3";
						$this->set_cart($items);
					}
					
					}

		}
		
		return TRUE;
	}
	
	public function out_of_stock($item_id, $item_location)
	{
		//make sure item exists		
		if($item_id != -1)
		{
			$item_quantity = $this->CI->Item_quantity->get_item_quantity($item_id, $item_location)->quantity;
			$quantity_added = $this->get_quantity_already_added($item_id, $item_location);
			//echo "Ac Stock:" .$item_quantity;
			//echo " Add Qn:" .$quantity_added." ";
			if($item_quantity - $quantity_added < 0)
			{
				return $this->CI->lang->line('sales_quantity_less_than_zero');
			}
			elseif($item_quantity - $quantity_added < $this->CI->Item->get_info_by_id_or_number($item_id)->reorder_level)
			{
				return $this->CI->lang->line('sales_quantity_less_than_reorder_level');
			}
		}

		return '';
	}


	public function out_of_stock_lock($item_id, $item_location,$quantity_added)
	{
		//make sure item exists		
		if($item_id != -1)
		{
			$item_quantity = $this->CI->Item_quantity->get_item_quantity($item_id, $item_location)->quantity;
			//$quantity_added = $this->get_quantity_already_added($item_id, $item_location);
			//echo "Ac Stock l:" .$item_quantity;
			//echo " Add Qn l:" .$quantity_added." ";
			if($item_quantity - $quantity_added < 0)
			{
				return $this->CI->lang->line('sales_quantity_less_than_zero');
			}
			elseif($item_quantity - $quantity_added < $this->CI->Item->get_info_by_id_or_number($item_id)->reorder_level)
			{
				//return $this->CI->lang->line('sales_quantity_less_than_reorder_level');
				return '1';
			}
		}

		return '1';
	}
	
	public function get_quantity_already_added($item_id, $item_location)
	{
		$items = $this->get_cart();
		$quanity_already_added = 0;
		foreach($items as $item)
		{
			if($item['item_id'] == $item_id && $item['item_location'] == $item_location)
			{
				$quanity_already_added+=$item['quantity'];
			}
		}
		
		return $quanity_already_added;
	}
	
	public function get_item_id($line_to_get)
	{
		$items = $this->get_cart();

		foreach($items as $line=>$item)
		{
			if($line == $line_to_get)
			{
				return $item['item_id'];
			}
		}
		
		return -1;
	}

	public function edit_item($line, $description, $serialnumber, $quantity, $discount, $price, $boxes, $pallets,$item_adicion)
	{
		

		$items = $this->get_cart();

		
		if(isset($items[$line]))	
		{

			$line = &$items[$line];
			//Preparar para modificar precio
			$item_number_line = $this->CI->Item->get_item_number($line['item_id']);
			$item_info = $this->CI->Item->get_info_by_id_or_number_custom($line['item_id']);
			$item_id = $item_info->item_id;
			//Traer la lista de precios
			$price_id = $item_info->custom1;
			$unit_box = $item_info->custom2;
			$unit_pallet = $item_info->custom4;
			//echo $item_number_line;
			//die();
			$price_quantity = 0;
			$pricekeys = array();

			if ($pallets > 0){
				$quantity = $unit_pallet * $pallets;	
			}

			if ($boxes > 0){
				$quantity = $unit_box * $boxes;	
			}



			/*Evitar editar para no superar stock*/

			if ($this->out_of_stock_lock($item_id, 1/*$item_location*/, $quantity)=='1') {
						# code...
						//echo "stock";
						$mensaje = "No se puede agregar el artículo a la venta";
						$this->CI->session->set_userdata('mensaje_personalizado', $mensaje);
						//$this->set_cart($items);
			}

			else{
						//echo "No stock";
						$mensaje = "La cantidad deseada es insuficiente. Por favor verifica el inventario";
						$this->CI->session->set_userdata('mensaje_personalizado', $mensaje);
						//return FALSE;//Habilitar al tener stock
			}




			//echo "ID ".$item_id." precio ".$price_id."<br>";
			
			//Si se edita un precio se encontrara esa edicion comparando el precio cargado con los precios de la lista
			//$items=$this->Item->get_prices_items($price_id);
			//echo "price id ".$price_id."<br>";
			$price_list=$this->CI->Item->get_unit_prices($price_id);

			//recorrer la lista de precios para comparar si se edito un precio
			$price_edit = 'S';
			//echo "a ver price_id  $price_id<br>";
			$price_param = $price;
			//var_dump($price_list);
			foreach ($price_list as $pricelist) 
			{
				//echo "precio ".$pricelist['item_unit_price']."!=".$price."<br>";
	    		if ($pricelist['item_unit_price']==$price)
	    		{
	    			//echo "mismo precio";
	    			if($pricelist['quantity_purchased']<=$quantity ){
	    				$price_edit = 'N';	
	    			}		
	    			//echo "coincide <br>";
	    		}
			}

			
			//Para modificar precios si se edita un item
			
			foreach($items as $item)
			{
				
				//Cargar en un array los que tienen lista de precio igual y guardar las cantidades

				$price_item=$this->CI->Item->get_price_id($item['item_id']);
				//echo "price item ".$price_item."<br>";
				if ($price_item == $price_id )
				{
					
					array_push($pricekeys, $item['line']);	
					if ($item['item_id'] != $item_id )
					{
						$price_quantity = bcadd($price_quantity, $items[$item['line']]['quantity']);	
						
					}	
					
				}
				//echo $item['description']." ".$item_info->description."<br>";
			}
			
			

			$sum_quant = bcadd($price_quantity, $quantity);
			//echo "cants ".$price_quantity." cant ".$quantity." suma ".$sum_quant."<br>";

			$price_temp = $this->CI->Item->price_calc($item_id, $sum_quant);
			$price = $price_temp;

			if ($price_edit=='S') {
				//echo "edita precio";
				$price = $price_param;	
			}
			
			//echo $price_temp;
			//die();
			//$line = &$items[$line];

			//Precio de costo para cliente 6107 Bodega Wil 2
	        $customer_id = $this->get_customer();//6107
	        if ($customer_id == 6107) {
	        	$price_unit = $item_info->cost_price;
	        	$price = $item_info->cost_price;
	        }
	        
			$line['description'] = $description;
			$line['serialnumber'] = $serialnumber;
			$line['quantity'] = $quantity;
			$line['discount'] = $discount;
			$line['price'] = $price;
			$line['pallets'] = $pallets;
			$line['boxes'] = $boxes;
			$line_total = $this->get_item_total($quantity, $price, $discount);
			//$line['total'] = $this->get_item_total($quantity, $price, $discount);
			$line['total'] = round($line_total);
			$line_discounted_total = $this->get_item_total($quantity, $price, $discount, TRUE);
			//$line['discounted_total'] = $this->get_item_total($quantity, $price, $discount, TRUE);
			$line['discounted_total'] = round($line_discounted_total);

			//recorrer los items con la misma lista de precios y modificar el precio segun lo calculado
			/*NO RECORRER PRECIO DESDE FICHA*/
			/*foreach ($pricekeys as $pricekey) 
			{
	    		$linea = &$items[$pricekey];

	    		if ($item_adicion==0) {
	    			//para modificar el precio segun cantidad
	            	$linea['price'] = $price;
					$linea['total'] = $this->get_item_total($linea['quantity'], $price, $linea['discount']);
					$linea['discounted_total'] = $this->get_item_total($linea['quantity'], $price, $linea['discount'], TRUE);
	    		}
	            
			}*/

			$this->set_cart($items);


		}

		//return FALSE;
		return TRUE;
	}

	public function delete_item($line,$item_adicion)
	{
		
		$items = $this->get_cart();

		//Guardar la linea
		$lines = &$items[$line];
		//Preparar para modificar precio
		$item_number_line = $this->CI->Item->get_item_number($lines['item_id']);
		$item_info = $this->CI->Item->get_info_by_id_or_number_custom($lines['item_id']);
		$item_id = $item_info->item_id;
		//Traer la lista de precios
		$price_id = $item_info->custom1;
		$price_quantity = 0;
		$pricekeys = array();

		//Verificar si es producto adicional con *
		$item_number = $lines['item_number'];
		//echo $item_number;
		$item_adicion = 0;
			if (substr($item_number,0,1)=="*") {
							//echo "*";
				$item_adicion = 1;			
			}

		//Para modificar precios de items con igual lista de precios
		foreach($items as $item)
		{		
			//Cargar en un array los que tienen lista de precio igual y guardar las cantidades
			$price_item=$this->CI->Item->get_price_id($item['item_id']);
			//echo $price_item."-".$price_id." ";
			if ($price_item == $price_id )
			{	
			//echo "misma lista";	
				array_push($pricekeys, $item['line']);	
				if ($item['item_id'] != $item_id )
				{
					//echo "diferente item";
					$price_quantity = bcadd($price_quantity, $items[$item['line']]['quantity']);				
				}		
			}
		}
		$price_temp = $this->CI->Item->price_calc($item_id, $price_quantity);
		//recorrer los items con la misma lista de precios y modificar el precio segun lo calculado
		/*NO RECORRER PRECIO DESDE FICHA*/
		/*foreach ($pricekeys as $pricekey) 
		{
	    	$linea = &$items[$pricekey];
	    	if ($item_adicion==0) {
	    		//para modificar el precio segun cantidad
	        	$linea['price'] = $price_temp;
				$linea['total'] = $this->get_item_total($linea['quantity'], $price_temp, $linea['discount']);
				$linea['discounted_total'] = $this->get_item_total($linea['quantity'], $price_temp, $linea['discount'], TRUE);
	    	}
	        
		}*/

		//$items = $this->get_cart();
		unset($items[$line]);
		$this->set_cart($items);
	}

	public function return_entire_sale($receipt_sale_id)
	{
		//POS #
		$pieces = explode(' ', $receipt_sale_id);
		$sale_id = $pieces[1];

		$this->empty_cart();
		$this->remove_customer();

		foreach($this->CI->Sale->get_sale_items($sale_id)->result() as $row)
		{
			$this->add_item($row->item_id, -$row->quantity_purchased, $row->item_location, $row->discount_percent, $row->item_unit_price, $row->description, $row->serialnumber, TRUE);
		}

		$this->set_customer($this->CI->Sale->get_customer($sale_id)->person_id);
	}
	
	public function add_item_kit($external_item_kit_id, $item_location, $discount)
	{
		//KIT #
		$pieces = explode(' ', $external_item_kit_id);
		$item_kit_id = $pieces[1];
		$result = TRUE;
		
		foreach($this->CI->Item_kit_items->get_info($item_kit_id) as $item_kit_item)
		{
			$result &= $this->add_item($item_kit_item['item_id'], $item_kit_item['quantity'], $item_location, $discount);
		}
		
		return $result;
	}

	public function copy_entire_sale($sale_id)
	{
		$this->empty_cart();
		$this->remove_customer();

		foreach($this->CI->Sale->get_sale_items($sale_id)->result() as $row)
		{
			$this->add_item($row->item_id, $row->quantity_purchased, $row->item_location, $row->discount_percent, $row->item_unit_price, $row->description, $row->serialnumber, TRUE);
		}

		foreach($this->CI->Sale->get_sale_payments($sale_id)->result() as $row)
		{
			$this->add_payment($row->payment_type, $row->payment_amount);
		}

		$this->set_customer($this->CI->Sale->get_customer($sale_id)->person_id);
		$this->set_employee($this->CI->Sale->get_employee($sale_id)->person_id);
		$this->set_seller($this->CI->Sale->get_seller($sale_id)->person_id);
	}
	
	public function copy_entire_suspended_sale($sale_id)
	{
		$this->empty_cart();
		$this->remove_customer();

		foreach($this->CI->Sale_suspended->get_sale_items($sale_id)->result() as $row)
		{
			$this->add_item($row->item_id, $row->quantity_purchased, $row->item_location, $row->discount_percent, $row->item_unit_price, $row->description, $row->serialnumber);
		}
		foreach($this->CI->Sale_suspended->get_sale_payments($sale_id)->result() as $row)
		{
			$this->add_payment($row->payment_type, $row->payment_amount);
		}
		$suspended_sale_info = $this->CI->Sale_suspended->get_info($sale_id)->row();
		$this->set_customer($suspended_sale_info->person_id);
		$this->set_comment($suspended_sale_info->comment);
		$this->set_invoice_number($suspended_sale_info->invoice_number);
		$this->set_restaurant_table($suspended_sale_info->table_id);
	}

	public function clear_all()
	{
		$this->set_invoice_number_enabled(FALSE);
		$this->clear_mode();
		$this->empty_cart();
		$this->clear_comment();
		$this->clear_email_receipt();
		$this->clear_invoice_number();
		$this->clear_giftcard_remainder();
		$this->empty_payments();
		$this->remove_customer();
		$this->remove_seller();
		$this->clear_comment2();
	}
	
	public function is_customer_taxable()
	{
		$customer_id = $this->get_customer();
		$customer = $this->CI->Customer->get_info($customer_id);
		
		//Do not charge sales tax if we have a customer that is not taxable
		return $customer->taxable or $customer_id == -1;
	}

	public function get_taxes()
	{
		$taxes = array();

		//Do not charge sales tax if we have a customer that is not taxable
		if($this->is_customer_taxable())
		{
			foreach($this->get_cart() as $line => $item)
			{
				$tax_info = $this->CI->Item_taxes->get_info($item['item_id']);

				foreach($tax_info as $tax)
				{
					$name = to_tax_decimals($tax['percent']) . '% ' . $tax['name'];
					$tax_amount = $this->get_item_tax($item['quantity'], $item['price'], $item['discount'], $tax['percent']);

					if(!isset($taxes[$name]))
					{
						$taxes[$name] = 0;
					}

					$taxes[$name] = bcadd($taxes[$name], $tax_amount);
				}
			}
		}

		return $taxes;
	}

	public function apply_customer_discount($discount_percent)
	{	
		// Get all items in the cart so far...
		$items = $this->get_cart();
		
		foreach($items as &$item)
		{
			$quantity = $item['quantity'];
			$price = $item['price'];

			// set a new discount only if the current one is 0
			if($item['discount'] == 0)
			{
				$item['discount'] = $discount_percent;
				$item['total'] = $this->get_item_total($quantity, $price, $discount_percent);
				$item['discounted_total'] = $this->get_item_total($quantity, $price, $discount_percent, TRUE);
			}
		}

		$this->set_cart($items);
	}
	
	public function get_discount()
	{
		$discount = 0;
		foreach($this->get_cart() as $item)
		{
			if($item['discount'] > 0)
			{
				$item_discount = $this->get_item_discount($item['quantity'], $item['price'], $item['discount']);
				$discount = bcadd($discount, $item_discount);
			}
		}

		return $discount;
	}

	public function get_subtotal($include_discount = FALSE, $exclude_tax = FALSE)
	{
		return $this->calculate_subtotal($include_discount, $exclude_tax);
	}
	
	public function get_item_total_tax_exclusive($item_id, $quantity, $price, $discount_percentage, $include_discount = FALSE) 
	{
		$tax_info = $this->CI->Item_taxes->get_info($item_id);
		$item_price = $this->get_item_total($quantity, $price, $discount_percentage, $include_discount);
		// only additive tax here
		foreach($tax_info as $tax)
		{
			$tax_percentage = $tax['percent'];
			$item_price = bcsub($item_price, $this->get_item_tax($quantity, $price, $discount_percentage, $tax_percentage));
		}
		
		return $item_price;
	}
	
	public function get_item_total($quantity, $price, $discount_percentage, $include_discount = FALSE)  
	{
		$total = bcmul($quantity, $price);
		if($include_discount)
		{
			$discount_amount = $this->get_item_discount($quantity, $price, $discount_percentage);

			return bcsub(round($total), $discount_amount);//redondeo por item
		}

		return round($total);//redondeo por item
	}
	
	public function get_item_discount($quantity, $price, $discount_percentage)
	{
		$total = bcmul($quantity, $price);
		$discount_fraction = bcdiv($discount_percentage, 100);

		return bcmul($total, $discount_fraction);
	}
	
	public function get_item_tax($quantity, $price, $discount_percentage, $tax_percentage) 
	{
		$price = $this->get_item_total($quantity, $price, $discount_percentage, TRUE);
		if($this->CI->config->config['tax_included'])
		{
			$tax_fraction = bcadd(100, $tax_percentage);
			$tax_fraction = bcdiv($tax_fraction, 100);
			$price_tax_excl = bcdiv($price, $tax_fraction);

			return bcsub($price, $price_tax_excl);
		}
		$tax_fraction = bcdiv($tax_percentage, 100);

		return bcmul($price, $tax_fraction);
	}

	public function calculate_subtotal($include_discount = FALSE, $exclude_tax = FALSE) 
	{
		$subtotal = 0;
		foreach($this->get_cart() as $item)
		{
			if($exclude_tax && $this->CI->config->config['tax_included'])
			{
				$subtotal = bcadd($subtotal, $this->get_item_total_tax_exclusive($item['item_id'], $item['quantity'], $item['price'], $item['discount'], $include_discount));
			}
			else 
			{
				$subtotal = bcadd($subtotal, $this->get_item_total($item['quantity'], $item['price'], $item['discount'], $include_discount));
			}
		}

		return $subtotal;
	}

	public function get_total()
	{
		$total = $this->calculate_subtotal(TRUE);		
		if(!$this->CI->config->config['tax_included'])
		{
			foreach($this->get_taxes() as $tax)
			{
				$total = bcadd($total, $tax);
			}
		}

		return $total;
	}
	/*BOTONERA POS*/

	public function get_category_pos()
	{
		if(!$this->CI->session->userdata('category_pos'))
		{
			$this->set_category_pos('LOMITOS');
		}

		return $this->CI->session->userdata('category_pos');
	}

	public function set_category_pos($category)
	{
		$this->CI->session->set_userdata('category_pos', $category);
	}

	public function clear_category_pos()
	{
		$this->CI->session->unset_userdata('category_pos');
	}

	/*FIN BOTONERA POS*/

	/*BOTONERA MESAS*/

	public function get_restaurant_table()
	{
		if(!$this->CI->session->userdata('restaurant_table'))
		{
			$this->set_restaurant_table('1');
		}

		return $this->CI->session->userdata('restaurant_table');
	}

	public function set_restaurant_table($table)
	{
		$this->CI->session->set_userdata('restaurant_table', $table);
	}

	public function clear_restaurant_table()
	{
		$this->CI->session->unset_userdata('restaurant_table');
	}

	/*FIN BOTONERA POS*/

	public function set_turno($turno) 
	{
		$this->CI->session->set_userdata('sales_turno', $turno);
	}
    public function get_turno() 
	{
		// avoid returning a NULL that results in a 0 in the comment if nothing is set/available
		$comment = $this->CI->session->userdata('sales_turno');

    	return empty($comment) ? '00' : $comment;
	}

	public function get_comment2() 
	{
		// avoid returning a NULL that results in a 0 in the comment if nothing is set/available
		$comment2 = $this->CI->session->userdata('sales_comment2');

    	return empty($comment2) ? '' : $comment2;
	}

	public function set_comment2($comment2) 
	{
		$this->CI->session->set_userdata('sales_comment2', $comment2);
	}

	public function clear_comment2() 	
	{
		$this->CI->session->unset_userdata('sales_comment2');
	}
}

?>
